/*
 This program demostrates binding wingets to queries.
 wxPg::Row_binding is used for displaying a row in widgets.
 wxPg::List_binding is used to fill list widgets (such as wxListBox) with the
    values from single column.
*/

#include<wx/wx.h>
#include<connectdlg/connectdlg.h>
#include<wxPg/wxPg.h>


//Application class
class
Ctrlbind_demo : public wxApp{
public:
    virtual bool
    OnInit();
};

IMPLEMENT_APP(Ctrlbind_demo)


//Program frame
class
Ctrlbind_demo_frame : public wxFrame{
public:
    Ctrlbind_demo_frame();
private:
    void
    connect_click(wxCommandEvent&);
    void
    next_click(wxCommandEvent&);
    void
    prev_click(wxCommandEvent&);

    bool
    query_open_(wxPg::Query &q, const wxString &sql);

    //A database connection
    wxPg::Database
    db_;

    //A query class for large queries
    wxPg::Query_browser
    query_;

    //A simpler query class for small queries or lower level work
    wxPg::Query
    align_query_, storage_query_;

    //A binding class for filling list widgets
    wxPg::List_binding
    align_binding_, storage_binding_;

    //A helper class for managing multiple lindings at once
    wxPg::UI_manager
    manager_;

    //A binding class for displaying one row in widgets
    wxPg::Row_binding
    binding_;

    DECLARE_EVENT_TABLE()
};


enum{
    ID_BUTTON_CONNECT = wxID_HIGHEST + 1,
    ID_BUTTON_NEXT,
    ID_BUTTON_PREV
};


bool
Ctrlbind_demo::OnInit(){
    Ctrlbind_demo_frame *win = new Ctrlbind_demo_frame;
    win->Show();
    this->SetTopWindow(win);
    return true;
}


/*
 Constructor for frame.
 All queries require a database connection.
 We pass pointer db_ to all query objects.
*/
Ctrlbind_demo_frame::Ctrlbind_demo_frame()
: wxFrame(NULL, -1, _("Control binding demo")), query_(&db_),
align_query_(&db_), storage_query_(&db_)
{
    wxPanel *panel = new wxPanel(this, -1);
    wxFlexGridSizer *sizer = new wxFlexGridSizer(3, 7);
    sizer->AddGrowableRow(6);

    wxButton *button = new wxButton(panel, ID_BUTTON_CONNECT, _("Connect"));
    sizer->Add(button);
    button = new wxButton(panel, ID_BUTTON_PREV, _("Prev"));
    sizer->Add(button, 0, wxALIGN_RIGHT);
    button = new wxButton(panel, ID_BUTTON_NEXT, _("Next"));
    sizer->Add(button);

    wxStaticText *label = new wxStaticText(panel, -1, _("typname"));
    sizer->Add(label, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
    wxTextCtrl *typname = new wxTextCtrl(panel, -1);
    sizer->Add(typname, 0, wxSHAPED|wxALIGN_CENTER_VERTICAL);
    sizer->AddSpacer(0);
    label = new wxStaticText(panel, -1, _("typarray"));
    sizer->Add(label, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
    wxSpinCtrl *typarray = new wxSpinCtrl(panel, -1);
    typarray->SetRange(0, 10000);
    sizer->Add(typarray, 0, wxALIGN_CENTER_VERTICAL);
    sizer->AddSpacer(0);
    sizer->AddSpacer(0);
    wxCheckBox *typbyval = new wxCheckBox(panel, -1, _("typbyval"));
    sizer->Add(typbyval);
    sizer->AddSpacer(0);
    sizer->AddSpacer(0);
    sizer->AddSpacer(0);
    sizer->AddSpacer(0);
    label = new wxStaticText(panel, -1, _("typalign"));
    sizer->Add(label, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
    label = new wxStaticText(panel, -1, _("typstorage"));
    sizer->Add(label, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);
    sizer->AddSpacer(0);
    wxListBox *typalign = new wxListBox(panel, -1);
    sizer->Add(typalign, 0, wxEXPAND);
    wxComboBox *typstorage = new wxComboBox(panel, -1);
    sizer->Add(typstorage, 0, wxSHAPED);

    
    panel->SetSizer(sizer);
    panel->Fit();

    /*
     Each binding requires some data source.
     wxPg::Row_binding requires wxPg::Data_row_provider descendant.
     wxPg::List_binding requires wxPg::Result_data_provider descendant.
    */
    this->binding_.data(&this->query_);
    this->align_binding_.data(&this->align_query_);
    this->storage_binding_.data(&this->storage_query_);

    /*
     We can add several bindings to a wxPg::UI_manger,
     and update all widgets, binded to these binding later
     with a single call.
    */
    this->manager_.add(&this->binding_);
    this->manager_.add(&this->align_binding_);
    this->manager_.add(&this->storage_binding_);
    
    /*
     Bind widgets, to display a certain values in a row.
     For list widgets (such as wxListBox) such binding changes
     selected item.
     For other widgets their values are changed.
    */
    wxPG_BIND_CONTROLS(&this->binding_)
        wxPG_BIND_TEXT_CTRL(typname, wxT("typname"))
        wxPG_BIND_SPIN_CTRL(typarray, wxT("typarray"))
        wxPG_BIND_CHECK_BOX(typbyval, wxT("typbyval"))
        wxPG_BIND_LIST_ITEM(typalign, wxT("typalign"))
        wxPG_BIND_LIST_ITEM(typstorage, wxT("typstorage"))
    wxPG_BINDING_END()

    //List binding fill list widget with values from one column
    wxPG_BIND_LIST_CONTROLS(&this->align_binding_)
        wxPG_BIND_LIST(typalign, wxT("typalign"))
    wxPG_BINDING_END()

    wxPG_BIND_LIST_CONTROLS(&this->storage_binding_)
        wxPG_BIND_LIST(typstorage, wxT("typstorage"))
    wxPG_BINDING_END()
}


bool
Ctrlbind_demo_frame::query_open_(wxPg::Query &q, const wxString &sql){
    if(!q.open(sql)){
        wxMessageBox(q.last_err());
        this->db_.connect(false);
        return false;
    }

    return true;
}


void
Ctrlbind_demo_frame::connect_click(wxCommandEvent&){
    Connect_dlg dlg(this);
    if(dlg.ShowModal())
        return;
    this->db_.connection_info(dlg.conninfo());
    if(!this->db_.connect()){
        wxMessageBox(_("Failed to connect!"));
        return;
    }

    /*
     Open queries.
     Note, that for wxPg::Query we need manually fetch rows, since it's a
        lower level query class.
    */
    if(!this->query_open_(this->align_query_,
                          wxT("select distinct typalign from pg_type")))
        return;
    this->align_query_.fetch(0, wxPg::Query::FM_FORWARD_ALL);
    if(!this->query_open_(this->storage_query_,
                          wxT("select distinct typstorage from pg_type")))
        return;
    this->storage_query_.fetch(0, wxPg::Query::FM_FORWARD_ALL);
    if(!this->query_open_(this->query_, wxT("select * from pg_type")))
        return;

    this->manager_.update_UI();
}


void
Ctrlbind_demo_frame::next_click(wxCommandEvent&){
    this->query_.next();
    //use binding_ directly, to avoid overhead
    this->binding_.update_UI();
}


void
Ctrlbind_demo_frame::prev_click(wxCommandEvent&){
    this->query_.prev();
    //use binding_ directly, to avoid overhead
    this->binding_.update_UI();
}


BEGIN_EVENT_TABLE(Ctrlbind_demo_frame, wxFrame)
    EVT_BUTTON(ID_BUTTON_CONNECT, Ctrlbind_demo_frame::connect_click)
    EVT_BUTTON(ID_BUTTON_NEXT, Ctrlbind_demo_frame::next_click)
    EVT_BUTTON(ID_BUTTON_PREV, Ctrlbind_demo_frame::prev_click)
END_EVENT_TABLE()
