/*
 A demo program, demonstrating wxPg::Grid.
 Program show, how wxPg::Grid can display entire result of SELECT statement,
 or only some columns of it.
 wxPg::Grid is designed to be able to display large result (a lot of rows).
*/

#include<wx/wx.h>
#include<connectdlg/connectdlg.h>
#include<wxPg/wxPg.h>
#include<wxPg/widgets.h>

//Application class
class
Grid_demo : public wxApp{
public:
    virtual bool
    OnInit();
};

IMPLEMENT_APP(Grid_demo)


enum{
    ID_BUTTON_CONNECT = wxID_HIGHEST + 1,
    ID_BUTTON_SELECT,
    ID_BUTTON_UPDATE_COLS
};

//A program frame
class
Grid_demo_frame : public wxFrame{
public:
    Grid_demo_frame();
private:
    void
    connect_click(wxCommandEvent&);

    void
    select_click(wxCommandEvent&);

    void
    update_cols_click(wxCommandEvent&);

    //A database connection
    wxPg::Database
    db_;

    //A grid
    wxPg::Grid*
    grid_;

    wxTextCtrl*
    sql_;

    wxListBox*
    col_list_;

    DECLARE_EVENT_TABLE()
};


bool
Grid_demo::OnInit(){
    Grid_demo_frame *win = new Grid_demo_frame();
    win->Show();
    this->SetTopWindow(win);
    return true;
}


Grid_demo_frame::Grid_demo_frame()
    : wxFrame(NULL, -1, _("Grid demo"))
{
    wxPanel *panel = new wxPanel(this, -1);
    wxFlexGridSizer *sizer = new wxFlexGridSizer(2, 2, 0, 0);
    sizer->AddGrowableRow(1);
    sizer->AddGrowableCol(1);

    wxButton *button = new wxButton(panel, ID_BUTTON_CONNECT, _("Connect"));
    sizer->Add(button);
    wxFlexGridSizer *sizer2 = new wxFlexGridSizer(1, 2, 0, 0);
    sizer2->AddGrowableCol(1);
    button = new wxButton(panel, ID_BUTTON_SELECT, _("Run"));
    sizer2->Add(button);
    this->sql_ = new wxTextCtrl(panel, -1, _("select * from pg_type"));
    sizer2->Add(this->sql_, 0, wxEXPAND);
    sizer->Add(sizer2, 0, wxEXPAND);

    sizer2 = new wxFlexGridSizer(2, 1, 0, 0);
    sizer2->AddGrowableRow(0);
    this->col_list_ = new wxListBox(panel, -1, wxDefaultPosition, wxDefaultSize,
                                    0, NULL, wxLB_MULTIPLE);
    sizer2->Add(this->col_list_, 0, wxEXPAND);
    button = new wxButton(panel, ID_BUTTON_UPDATE_COLS, _("Show selected"));
    sizer2->Add(button);
    sizer->Add(sizer2, 0, wxEXPAND);

    /*
     wxPg::Grid is wxGrid descendant.
     The main differrences are:
        it needs database connection
        it uses wxPg::Grid_table as it's table
    */
    this->grid_ = new wxPg::Grid(panel, -1);
    this->grid_->database(&this->db_);
    sizer->Add(this->grid_, 0, wxEXPAND);

    panel->SetSizer(sizer);
}


void
Grid_demo_frame::connect_click(wxCommandEvent&){
    Connect_dlg dlg(this);
    if(!dlg.ShowModal()){
        this->db_.connection_info(dlg.conninfo());
        if(!this->db_.connect())
            wxMessageBox(_("Failed to connect!"));
    }
}


void
Grid_demo_frame::select_click(wxCommandEvent&){
    if(!this->db_.connected())
        return;

    /*
     wxPg::Grid uses wxPg::Grid_table as its table.
     wxPg::Grid_table has internal query for executing SQL statements.
     */
    if(!this->grid_->open(this->sql_->GetValue()))
        wxMessageBox(this->grid_->last_err());
    else{
        this->col_list_->Clear();
        wxPg::Query_browser &query = static_cast<wxPg::Grid_table*>(
                this->grid_->GetTable())->query();
        for(int i = 0; i < query.cols(); ++i)
            this->col_list_->Append(query.col_name(i));
    }
}


void
Grid_demo_frame::update_cols_click(wxCommandEvent&){
    wxArrayInt selections;

    /*
     wxPg::Grid_table has methods for selecting, which columns to display.
     By default it displays all columns, if none are binded.
    */
    wxPg::Grid_table *table = static_cast<wxPg::Grid_table*>(this->grid_->GetTable());
    if(!table)
        return;
    this->col_list_->GetSelections(selections);
    table->unbind();
    for(unsigned i = 0; i < selections.Count(); ++i)
        table->bind(this->col_list_->GetString(selections[i]));

    //After modifying a table, we need to re-set it to grid and do a replain.
    this->grid_->SetTable(NULL);
    this->grid_->SetTable(table);
    this->grid_->ForceRefresh();
}


BEGIN_EVENT_TABLE(Grid_demo_frame, wxFrame)
    EVT_BUTTON(ID_BUTTON_CONNECT, Grid_demo_frame::connect_click)
    EVT_BUTTON(ID_BUTTON_SELECT, Grid_demo_frame::select_click)
    EVT_BUTTON(ID_BUTTON_UPDATE_COLS, Grid_demo_frame::update_cols_click)
END_EVENT_TABLE()
