/* binding_ctrl.cpp - helper functions for binding wxWindow to
    wxPg::Row_binding

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<wx/calctrl.h>
#include<wx/combobox.h>
#include<wx/combo.h>
#include<wx/richtext/richtextctrl.h>
#include<wx/textctrl.h>
#include<wx/spinctrl.h>
#include<wx/stattext.h>
#include<wx/hyperlink.h>
#include<wx/datectrl.h>
#include<wx/checkbox.h>
#include<wx/gauge.h>
#include<wx/spinbutt.h>
#include<wx/radiobox.h>
#include<wx/vlbox.h>
#include<wx/html/htmlwin.h>
#include<wx/bmpcbox.h>
#include<wx/choice.h>
#include<wx/htmllbox.h>
#include<wx/listbox.h>
#include<wx/odcombo.h>
#include<wxPg/binding/binding_ctrl.h>


wxPg::wxWindow_updater::wxWindow_updater(wxWindow *win)
    : win_(win)
{}



wxPg::wxWindow_def_updater::wxWindow_def_updater(wxWindow *win,
                                                 const wxString &field)
: wxWindow_updater(win), field_(field)
{}



wxPg::wxCalendarCtrl_updater::wxCalendarCtrl_updater(wxCalendarCtrl *calendar,
                                                     const wxString &field)
: wxWindow_def_updater(calendar, field)
{}


void
wxPg::wxCalendarCtrl_updater::operator()(Data_row_provider *data){
    static_cast<wxCalendarCtrl*>(this->window())->SetDate(
            data->get_date(this->field()));
}


void
wxPg::wxCalendarCtrl_updater::operator()(Updatable_row *row){
    row->set_date(this->field(),
                  static_cast<wxCalendarCtrl*>(this->window())->GetDate());
}




wxPg::wxComboBox_updater::wxComboBox_updater(wxComboBox *combo,
                                             const wxString &field)
: wxWindow_def_updater(combo, field)
{}


void
wxPg::wxComboBox_updater::operator()(Data_row_provider *data){
    static_cast<wxComboBox*>(this->window())->SetValue(
            data->get_str(this->field()));
}


void
wxPg::wxComboBox_updater::operator()(Updatable_row *row){
    row->set_value(this->field(),
                 static_cast<wxComboBox*>(this->window())->GetValue());
}




wxPg::wxComboCtrl_updater::wxComboCtrl_updater(wxComboCtrl *combo,
                                               const wxString &field)
: wxWindow_def_updater(combo, field)
{}


void
wxPg::wxComboCtrl_updater::operator()(Data_row_provider *data){
    static_cast<wxComboCtrl*>(this->window())->SetValue(
            data->get_str(this->field()));
}


void
wxPg::wxComboCtrl_updater::operator()(Updatable_row *row){
    row->set_value(this->field(),
                   static_cast<wxComboCtrl*>(this->window())->GetValue());
}




wxPg::wxRichTextCtrl_updater::wxRichTextCtrl_updater(wxRichTextCtrl *text,
                                                     const wxString &field)
: wxWindow_def_updater(text, field)
{}


void
wxPg::wxRichTextCtrl_updater::operator()(Data_row_provider *data){
    static_cast<wxRichTextCtrl*>(this->window())->SetValue(
            data->get_str(this->field()));
}


void
wxPg::wxRichTextCtrl_updater::operator()(Updatable_row *row){
    row->set_value(this->field(),
                   static_cast<wxRichTextCtrl*>(this->window())->GetValue());
}




wxPg::wxTextCtrl_updater::wxTextCtrl_updater(wxTextCtrl *text,
                                             const wxString &field)
: wxWindow_def_updater(text, field)
{}


void
wxPg::wxTextCtrl_updater::operator()(Data_row_provider *data){
    static_cast<wxTextCtrl*>(this->window())->SetValue(
            data->get_str(this->field()));
}


void
wxPg::wxTextCtrl_updater::operator()(Updatable_row *row){
    row->set_value(this->field(),
                   static_cast<wxTextCtrl*>(this->window())->GetValue());
}




wxPg::wxSpinCtrl_updater::wxSpinCtrl_updater(wxSpinCtrl *spin,
                                             const wxString &field)
: wxWindow_def_updater(spin, field)
{}


void
wxPg::wxSpinCtrl_updater::operator()(Data_row_provider *data){
    static_cast<wxSpinCtrl*>(this->window())->SetValue(
            data->get_int(this->field()));
}


void
wxPg::wxSpinCtrl_updater::operator()(Updatable_row *row){
    row->set_value(this->field(),
                   static_cast<wxSpinCtrl*>(this->window())->GetValue());
}




wxPg::wxStaticText_updater::wxStaticText_updater(wxStaticText *text,
                                                 const wxString &field)
: wxWindow_def_updater(text, field)
{}


void
wxPg::wxStaticText_updater::operator()(Data_row_provider *data){
    static_cast<wxStaticText*>(this->window())->SetLabel(
            data->get_str(this->field()));
}




wxPg::wxHyperlinkCtrl_updater::wxHyperlinkCtrl_updater(
        wxHyperlinkCtrl *hyperlink, const wxString &field)
: wxWindow_def_updater(hyperlink, field)
{}


void
wxPg::wxHyperlinkCtrl_updater::operator()(Data_row_provider *data){
    static_cast<wxHyperlinkCtrl*>(this->window())->SetURL(
            data->get_str(this->field()));
}




wxPg::wxHyperlinkCtrl_updater_wlabel::wxHyperlinkCtrl_updater_wlabel(
        wxHyperlinkCtrl *hyperlink, const wxString &link_field,
        const wxString &label_field)
: wxPg::wxHyperlinkCtrl_updater(hyperlink, link_field),
    label_field_(label_field)
{}


void
wxPg::wxHyperlinkCtrl_updater_wlabel::operator()(Data_row_provider *data){
    static_cast<wxHyperlinkCtrl*>(this->window())->SetLabel(
            data->get_str(this->label_field()));
    this->wxHyperlinkCtrl_updater::operator()(data);
    static_cast<wxHyperlinkCtrl*>(this->window())->Refresh();
}





wxPg::wxDatePickerCtrl_updater::wxDatePickerCtrl_updater(
        wxDatePickerCtrl *date_ctrl, const wxString &field)
: wxWindow_def_updater(date_ctrl, field)
{}


void
wxPg::wxDatePickerCtrl_updater::operator()(Data_row_provider *data){
    static_cast<wxDatePickerCtrl*>(this->window())->SetValue(
            data->get_date(this->field()));
}


void
wxPg::wxDatePickerCtrl_updater::operator()(Updatable_row *row){
    wxDateTime date = static_cast<wxDatePickerCtrl*>(
            this->window())->GetValue();
    if(date.IsValid())
        row->set_date(this->field(), date);
    else row->set_null(this->field());
}




wxPg::wxCheckBox_updater::wxCheckBox_updater(wxCheckBox *check_box,
                                             const wxString &field)
: wxWindow_def_updater(check_box, field)
{}


void
wxPg::wxCheckBox_updater::operator()(Data_row_provider *data){
    static_cast<wxCheckBox*>(this->window())->SetValue(
            data->get_bool(this->field()));
}


void
wxPg::wxCheckBox_updater::operator()(Updatable_row *row){
    row->set_bool(this->field(),
                  static_cast<wxCheckBox*>(this->window())->GetValue());
}




wxPg::wxGauge_updater::wxGauge_updater(wxGauge *gauge, const wxString &field)
: wxWindow_def_updater(gauge, field)
{}


void
wxPg::wxGauge_updater::operator()(Data_row_provider *data){
    static_cast<wxGauge*>(this->window())->SetValue(
            data->get_int(this->field()));
}




wxPg::wxSpinButton_updater::wxSpinButton_updater(wxSpinButton *spin,
                                                 const wxString &field)
: wxWindow_def_updater(spin, field)
{}


void
wxPg::wxSpinButton_updater::operator()(Data_row_provider *data){
    static_cast<wxSpinButton*>(this->window())->SetValue(
            data->get_int(this->field()));
}


void
wxPg::wxSpinButton_updater::operator()(Updatable_row *row){
    row->set_value(this->field(),
                   static_cast<wxSpinButton*>(this->window())->GetValue());
}




wxPg::wxItemContainer_updater::wxItemContainer_updater(wxBitmapComboBox *combo,
                                                       const wxString &field)
: wxWindow_def_updater(combo, field)
{}


wxPg::wxItemContainer_updater::wxItemContainer_updater(wxChoice *choice,
                                                       const wxString &field)
: wxWindow_def_updater(choice, field)
{}


wxPg::wxItemContainer_updater::wxItemContainer_updater(wxComboBox *combo,
                                                       const wxString &field)
: wxWindow_def_updater(combo, field)
{}


wxPg::wxItemContainer_updater::wxItemContainer_updater(
        wxSimpleHtmlListBox *list, const wxString &field)
: wxWindow_def_updater(list, field)
{}


wxPg::wxItemContainer_updater::wxItemContainer_updater(wxListBox *list,
                                                       const wxString &field)
: wxWindow_def_updater(list, field)
{}


wxPg::wxItemContainer_updater::wxItemContainer_updater(
        wxOwnerDrawnComboBox *combo, const wxString &field)
: wxWindow_def_updater(combo, field)
{}


wxPg::wxItemContainer_updater::wxItemContainer_updater(wxRadioBox *radio_box,
                                                       const wxString &field)
: wxWindow_def_updater(radio_box, field)
{}


void
wxPg::wxItemContainer_updater::operator()(Data_row_provider *data){
    //need dynamic_cast because of multiple inheritance
    wxItemContainer *cont = dynamic_cast<wxItemContainer*>(this->window());
    if(cont){
        if(data->null(this->field()))
            cont->SetSelection(wxNOT_FOUND);
        else if(!cont->SetStringSelection(data->get_str(this->field())))
            cont->SetSelection(wxNOT_FOUND);
    }
}


void
wxPg::wxItemContainer_updater::operator()(Updatable_row *row){
    //need dynamic_cast because of multiple inheritance
    wxItemContainer *cont = dynamic_cast<wxItemContainer*>(this->window());
    if(cont){
        if(cont->GetSelection() == wxNOT_FOUND)
            row->set_null(this->field());
        else row->set_value(this->field(), cont->GetStringSelection());
    }
}




wxPg::wxRadioBox_updater::wxRadioBox_updater(wxRadioBox *box,
                                             const wxString &field)
: wxWindow_def_updater(box, field)
{}


void
wxPg::wxRadioBox_updater::operator()(Data_row_provider *data){
    static_cast<wxRadioBox*>(this->window())->SetSelection(
            data->get_int(this->field()));
}


void
wxPg::wxRadioBox_updater::operator()(Updatable_row *row){
    row->set_value(this->field(),
                   static_cast<wxRadioBox*>(this->window())->GetSelection());
}




wxPg::wxVListBox_updater::wxVListBox_updater(wxVListBox *box,
                                             const wxString &field)
: wxWindow_def_updater(box, field)
{}


void
wxPg::wxVListBox_updater::operator()(Data_row_provider *data){
    static_cast<wxVListBox*>(this->window())->SetSelection(
            data->get_int(this->field()));
}


void
wxPg::wxVListBox_updater::operator()(Updatable_row *row){
    int value = static_cast<wxVListBox*>(this->window())->GetSelection();
    if(value == wxNOT_FOUND)
        row->set_null(this->field());
    else row->set_value(this->field(), value);
}




wxPg::wxHtmlWindow_updater::wxHtmlWindow_updater(wxHtmlWindow *html,
                                                 const wxString &field)
: wxWindow_def_updater(html, field)
{}


void
wxPg::wxHtmlWindow_updater::operator()(Data_row_provider *data){
    static_cast<wxHtmlWindow*>(this->window())->SetPage(
            data->get_str(this->field()));
}

