/* grid_bind_def.cpp - class wxPg::Grid_bind_def for containing grid binding
    definitions.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<wxPg/binding/grid_bind_def.h>


wxPg::Grid_bind_def::Grid_bind_def(const std::map<int, wxString> &col_bindings)
    :col_bindings_(col_bindings)
{}


wxPg::Grid_bind_def::Grid_bind_def(const std::map<int, wxString> &col_bindings,
                                   const std::map<wxString, wxString> &labels)
    :col_bindings_(col_bindings), col_labels_(labels)
{}


void
wxPg::Grid_bind_def::bind(const wxString &field){
    if(this->col_bindings_.size()){
        int new_col_idx = (--(this->col_bindings_.end()))->first + 1;
        this->col_bindings_[new_col_idx] = field;
    }else this->col_bindings_[0] = field;
}


int
wxPg::Grid_bind_def::field_binding(const wxString &field)const{
    for(std::map<int, wxString>::const_iterator
            i = this->col_bindings_.begin();
            i != this->col_bindings_.end(); ++i)
        if(i->second == field)
            return i->first;
    return -1;
}


unsigned
wxPg::Grid_bind_def::col_count()const{
    if(this->col_bindings_.size())
        return (--(this->col_bindings_.end()))->first + 1;
    return 0;
}


const wxString
wxPg::Grid_bind_def::col_label(unsigned col_idx)const{
    std::map<int, wxString>::const_iterator
    column = this->col_bindings_.find(col_idx);

    if(column == this->col_bindings_.end())
        return wxEmptyString;

    std::map<wxString, wxString>::const_iterator
    it = this->col_labels_.find(column->second);

    if(it != this->col_labels_.end())
        return it->second;

    return column->second;
}
