/* list_binding.cpp - binding wxItemContainer to contain the values of some
    field in wxPg::Result_data

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/binding/list_binding.h>


wxPg::List_binding::List_binding()
    :data_(NULL)
{}


wxPg::List_binding::List_binding(wxPg::Result_data_provider *data)
    :data_(data)
{}


void
wxPg::List_binding::bind(wxWindow_list_updater *updater){
    for(unsigned i = 0; i < this->ctrl_bindings_.size(); ++i)
        if(this->ctrl_bindings_[i]->window() == updater->window()){
            if(this->ctrl_bindings_[i]->delete_me())
                delete this->ctrl_bindings_[i];
            this->ctrl_bindings_[i] = updater;
            return;
        }

    this->ctrl_bindings_.push_back(updater);
}


void
wxPg::List_binding::unbind(wxWindow *w){
    for(std::vector<wxWindow_list_updater*>::iterator
            i = this->ctrl_bindings_.begin();
            i != this->ctrl_bindings_.end(); ++i)
        if((*i)->window() == w){
            if((*i)->delete_me())
                delete (*i);
            this->ctrl_bindings_.erase(i);
            break;
        }
}


void
wxPg::List_binding::update_UI(){
    if(!this->data())
        return;

    std::vector<wxWindow_list_updater*> by_row;
    by_row.reserve(this->ctrl_bindings_.size());
    for(unsigned i = 0; i < this->ctrl_bindings_.size(); ++i)
        if(this->ctrl_bindings_[i]->use_fo())
            (*this->ctrl_bindings_[i])(this->data());
        else by_row.push_back(this->ctrl_bindings_[i]);

    int current = this->data()->current();
    this->data()->current(0);
    int total_rows = this->data()->rows();
    for(unsigned i = 0; i < by_row.size(); ++i)
        by_row[i]->begin_update(this->data());
    for(int i = 0; i < total_rows; ++i){
        this->data()->current(i);
        for(unsigned j = 0; j < by_row.size(); ++j)
            by_row[j]->update_row(this->data());
    }
    this->data()->current(current);
    for(unsigned i = 0; i < by_row.size(); ++i)
        by_row[i]->end_update();
}
