/* row_binding.cpp - binding wxWindow to display value of some
    field in wxPg::Data_row

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<wxPg/binding/row_binding.h>


wxPg::Row_binding::Row_binding()
    :data_(NULL)
{}


wxPg::Row_binding::Row_binding(wxPg::Data_row_provider *data)
    :data_(data)
{}


wxPg::Row_binding::~Row_binding(){
    for(unsigned i = 0; i < this->ctrl_bindings_.size(); ++i)
        if(this->ctrl_bindings_[i]->delete_me())
            delete this->ctrl_bindings_[i];
    this->ctrl_bindings_.clear();
}


void
wxPg::Row_binding::bind(wxWindow_updater *updater){
    for(unsigned i = 0; i < this->ctrl_bindings_.size(); ++i)
        if(this->ctrl_bindings_[i]->window() == updater->window()){
            if(this->ctrl_bindings_[i]->delete_me())
                delete this->ctrl_bindings_[i];
            this->ctrl_bindings_[i] = updater;
            return;
        }

    this->ctrl_bindings_.push_back(updater);
}


void
wxPg::Row_binding::unbind(wxWindow *w){
    for(std::vector<wxWindow_updater*>::iterator
            i = this->ctrl_bindings_.begin();
            i != this->ctrl_bindings_.end(); ++i)
        if((*i)->window() == w){
            if((*i)->delete_me())
                delete (*i);
            this->ctrl_bindings_.erase(i);
            break;
        }
}


void
wxPg::Row_binding::update_UI(){
    if(!this->data_)
        return;
    for(unsigned i = 0; i < this->ctrl_bindings_.size(); ++i)
        (*this->ctrl_bindings_[i])(this->data());
}
