/* data_row_test.cpp - test program for wxPg::Cached_result.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<iostream>
#include<wx/wx.h>
#include<test_logger.h>
#include<testdb.h>
#include<test_logger.h>
#include<wxPg/db/query.h>
#include<getopt.h>
#include<cstdlib>
#include<wxPg/utils/type.h>


wxPg::Query
testee(&db);


int
test_cols,
test_types,
test_get_int,
test_get_float,
test_get_str,
test_get_date,
test_get_time,
test_get_date_time,
test_get_bool;


option
opts[] = {
    {"help", no_argument, NULL, 'h'},
    {"test-cols", no_argument, &test_cols, 1},
    {"test-types", no_argument, &test_types, 1},
    {"test-get_int", no_argument, &test_get_int, 1},
    {"test-get_float", no_argument, &test_get_float, 1},
    {"test-get_str", no_argument, &test_get_str, 1},
    {"test-get_date", no_argument, &test_get_date, 1},
    {"test-get_time", no_argument, &test_get_time, 1},
    {"test-get_date_time", no_argument, &test_get_date_time, 1},
    {"test-get_bool", no_argument, &test_get_bool, 1},
    {NULL, 0, NULL, 0}
};


void
print_help(){
    std::cout << "Test program for Data_row.\n"
              << "When no options are given, all tests are run.\n"
              << "Accepted options:\n"
              << "\t--help - print help and exit.\n"
              << "\t--test-cols - test column count and column names.\n"
              << "\t--test-types - test column types.\n"
              << "\t--test-get_int - test get_int() for each column.\n"
              << "\t--test-get_float - test get_float() for each column.\n"
              << "\t--test-get_str - test get_str() for each column.\n"
              << "\t--test-get_date - test get_date() for each column.\n"
              << "\t--test-get_time - test get_time() for each column.\n"
              << "\t--test-get_date_time - test get_date_time() for each column.\n"
              << "\t--test-get_bool - test get_bool() for each column.\n";
    std::exit(0);
}


void
parse_options(int argc, char **argv){
    int opt;
    while((opt = getopt_long(argc, argv, "", opts, NULL)) >= 0)
        if(opt == 'h')
            print_help();

    if(test_cols == 0 && test_types == 0 && test_get_int == 0 && test_get_float == 0
            && test_get_str == 0 && test_get_date == 0 && test_get_time == 0
            && test_get_date_time == 0 && test_get_bool == 0)
        test_cols = test_types = test_get_int = test_get_float = test_get_str
        = test_get_date = test_get_time = test_get_date_time = test_get_bool = 1;
}



bool
prepare_test_env(){
    if(!db_connect())
        return false;

    wxString create_table = wxT(
        "CREATE TABLE data_row_test( \
           id integer, \
           fnum double precision, \
           create_date date, \
           create_time time without time zone, \
           create_timestamp timestamp without time zone, \
           ok boolean, \
           title text, \
           null_int integer \
        );"
    );

    if(!db.run(create_table)){
        logger.log_error("Fatal error", "Failed to create test table.");
        return false;
    }

    bool ins_success = db.run(wxT("insert into data_row_test values( \
                                   1, 0.5, '2000-01-01', '10:10', '2000-01-01 10:10', \
                                   true, 'aaa', NULL)"));

    if(!ins_success)
        logger.log_error("Fatal error", "Failed to insert test data.");

    return ins_success;
}


void
drop_test_table(){
    db.run(wxT("drop table data_row_test"));
}


bool
check_col_idx(const wxString &col_name, int idx){
    if(testee.col_name(idx) != col_name){
        wxString msg = wxT("Expected col ");
        msg << idx << wxT(" to be ") << col_name
            << wxT(" found ") << testee.col_name(idx);
        logger.log_error("Test cols", msg.char_str());
        return false;
    }
    if(testee.col_idx(col_name) != idx){
        wxString msg = wxT("Expected col ");
        msg << col_name << wxT(" to be at index ") << idx
            << wxT(" found at ") << testee.col_idx(col_name);
        logger.log_error("Test cols", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_cols(){
    if(testee.cols() != 8){
        wxString msg = wxT("Expected 7 columns, got ");
        msg << testee.cols();
        logger.log_error("Test cols", msg.char_str());
        return false;
    }


    if(!check_col_idx(wxT("id"), 0))
        return false;
    if(!check_col_idx(wxT("fnum"), 1))
        return false;
    if(!check_col_idx(wxT("create_date"), 2))
        return false;
    if(!check_col_idx(wxT("create_time"), 3))
        return false;
    if(!check_col_idx(wxT("create_timestamp"), 4))
        return false;
    if(!check_col_idx(wxT("ok"), 5))
        return false;
    if(!check_col_idx(wxT("title"), 6))
        return false;
    if(!check_col_idx(wxT("null_int"), 7))
        return false;

    logger.log_success("Test cols");
    return true;
}


bool
check_field_type(int idx, const wxString &name, int type){
    if(testee.type_simple(idx) != type){
        wxString msg = wxT("Expected type ");
        msg << type << wxT(" for column ") << idx
            << wxT(" got ") << testee.type_simple(idx);
        logger.log_error("Test types", msg.char_str());
        return false;
    }
    if(testee.type_simple(name) != type){
        wxString msg = wxT("Expected type ");
        msg << type << wxT(" for column ") << name
            << wxT(" got ") << testee.type_simple(name);
        logger.log_error("Test types", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_types(){
    if(!check_field_type(0, wxT("id"), wxPg::TYPE_INT))
        return false;
    if(!check_field_type(1, wxT("fnum"), wxPg::TYPE_FLOAT))
        return false;
    if(!check_field_type(2, wxT("create_date"), wxPg::TYPE_DATE))
        return false;
    if(!check_field_type(3, wxT("create_time"), wxPg::TYPE_TIME))
        return false;
    if(!check_field_type(4, wxT("create_timestamp"), wxPg::TYPE_TIMESTAMP))
        return false;
    if(!check_field_type(5, wxT("ok"), wxPg::TYPE_BOOL))
        return false;
    if(!check_field_type(6, wxT("title"), wxPg::TYPE_STRING))
        return false;

    logger.log_success("Test types");
    return true;
}


bool
check_null(int idx, const wxString &name, const char *msg){
    if(!testee.null(idx)){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT(" to be NULL.");
        logger.log_error("Test get_int", msg.char_str());
        return false;
    }
    if(!testee.null(name)){
        wxString msg = wxT("Expected column ");
        msg << name << wxT(" to be NULL.");
        logger.log_error("Test get_int", msg.char_str());
        return false;
    }

    return true;
}


bool
check_int(int idx, const wxString &name, bool null, int value){
    if(null){
        if(!check_null(idx, name, "Test get_int"))
            return false;
        value = 0;
    }
    if(testee.get_int(idx) != value){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT(" to be equal to ") << value
            << wxT(" got ") << testee.get_int(idx);
        logger.log_error("Test get_int", msg.char_str());
        return false;
    }
    if(testee.get_int(name) != value){
        wxString msg = wxT("Expected column ");
        msg << name << wxT(" to be equal to ") << value
            << wxT(" got ") << testee.get_int(name);
        logger.log_error("Test get_int", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_get_int(){
    if(!check_int(0, wxT("id"), false, 1))
        return false;
    if(!check_int(1, wxT("fnum"), false, 0))
        return false;
    if(!check_int(2, wxT("create_date"), false, 2000))
        return false;
    if(!check_int(3, wxT("create_time"), false, 10))
        return false;
    if(!check_int(4, wxT("create_timestamp"), false, 2000))
        return false;
    if(!check_int(5, wxT("ok"), false, 1))
        return false;
    if(!check_int(6, wxT("title"), false, 0))
        return false;
    if(!check_int(7, wxT("null_int"), true, 0))
        return false;

    logger.log_success("Test get_int");
    return true;
}


bool
check_float(int idx, const wxString &name, bool null, double value){
    if(null){
        if(!check_null(idx, name, "Test get_float"))
            return false;
        value = 0;
    }
    if(testee.get_float(idx) != value){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT(" to be equal to ") << value
            << wxT(" got ") << testee.get_float(idx);
        logger.log_error("Test get_float", msg.char_str());
        return false;
    }
    if(testee.get_float(name) != value){
        wxString msg = wxT("Expected column ");
        msg << name << wxT(" to be equal to ") << value
            << wxT(" got ") << testee.get_float(name);
        logger.log_error("Test get_float", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_get_float(){
    if(!check_float(0, wxT("id"), false, 1))
        return false;
    if(!check_float(1, wxT("fnum"), false, 0.5))
        return false;
    if(!check_float(2, wxT("create_date"), false, 2000.0))
        return false;
    if(!check_float(3, wxT("create_time"), false, 10.0))
        return false;
    if(!check_float(4, wxT("create_timestamp"), false, 2000.0))
        return false;
    if(!check_float(5, wxT("ok"), false, 1.0))
        return false;
    if(!check_float(6, wxT("title"), false, 0.0))
        return false;
    if(!check_float(7, wxT("null_int"), true, 0.0))
        return false;

    logger.log_success("Test get_float");
    return true;
}


bool
check_str(int idx, const wxString &name, bool null, wxString value){
    if(null){
        if(!check_null(idx, name, "Test get_str"))
            return false;
        value = wxEmptyString;
    }
    if(testee.get_str(idx) != value){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT(" to be equal to ") << value
            << wxT(" got ") << testee.get_str(idx);
        logger.log_error("Test get_str", msg.char_str());
        return false;
    }
    if(testee.get_str(name) != value){
        wxString msg = wxT("Expected column ");
        msg << name << wxT(" to be equal to ") << value
            << wxT(" got ") << testee.get_str(name);
        logger.log_error("Test get_str", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_get_str(){
    if(!check_str(0, wxT("id"), false, wxT("1")))
        return false;
    if(!check_str(1, wxT("fnum"), false, wxT("0.5")))
        return false;
    if(!check_str(2, wxT("create_date"), false, wxT("2000-01-01")))
        return false;
    if(!check_str(3, wxT("create_time"), false, wxT("10:10:00")))
        return false;
    if(!check_str(4, wxT("create_timestamp"), false, wxT("2000-01-01 10:10:00")))
        return false;
    if(!check_str(5, wxT("ok"), false, wxT("t")))
        return false;
    if(!check_str(6, wxT("title"), false, wxT("aaa")))
        return false;
    if(!check_str(7, wxT("null_int"), true, wxEmptyString))
        return false;

    logger.log_success("Test get_str");
    return true;
}


bool
check_date(int idx, const wxString &name, bool null, wxDateTime value){
    if(null){
        if(!check_null(idx, name, "Test get_date"))
            return false;
        value = wxDateTime();
    }
    if(!testee.get_date(idx).IsSameDate(value)){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT(" to be equal to ") << value.FormatISODate()
            << wxT(" got ") << testee.get_date(idx).FormatISODate();
        logger.log_error("Test get_date", msg.char_str());
        return false;
    }
    if(!testee.get_date(name).IsSameDate(value)){
        wxString msg = wxT("Expected column ");
        msg << name << wxT(" to be equal to ") << value.FormatISODate()
            << wxT(" got ") << testee.get_date(name).FormatISODate();
        logger.log_error("Test get_date", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_get_date(){
    wxDateTime date, date_emp;
    date.ParseDate(wxT("2000-01-01"));

    if(!check_date(0, wxT("id"), false, date_emp))
        return false;
    if(!check_date(1, wxT("fnum"), false, date_emp))
        return false;
    if(!check_date(2, wxT("create_date"), false, date))
        return false;
    if(!check_date(3, wxT("create_time"), false, date_emp))
        return false;
    if(!check_date(4, wxT("create_timestamp"), false, date))
        return false;
    if(!check_date(5, wxT("ok"), false, date_emp))
        return false;
    if(!check_date(6, wxT("title"), false, date_emp))
        return false;
    if(!check_date(7, wxT("null_int"), true, date_emp))
        return false;

    logger.log_success("Test get_date");
    return true;
}


bool
check_time(int idx, const wxString &name, bool null, wxDateTime value){
    if(null){
        if(!check_null(idx, name, "Test get_time"))
            return false;
        value = wxDateTime();
    }
    if(!testee.get_time(idx).IsSameTime(value)){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT(" to be equal to ") << value.FormatISOTime()
            << wxT(" got ") << testee.get_time(idx).FormatISOTime();
        logger.log_error("Test get_time", msg.char_str());
        return false;
    }
    if(!testee.get_time(name).IsSameTime(value)){
        wxString msg = wxT("Expected column ");
        msg << name << wxT(" to be equal to ") << value.FormatISOTime()
            << wxT(" got ") << testee.get_time(name).FormatISOTime();
        logger.log_error("Test get_time", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_get_time(){
    wxDateTime time, time_emp;
    time.ParseTime(wxT("10:10"));

    if(!check_time(0, wxT("id"), false, time_emp))
        return false;
    if(!check_time(1, wxT("fnum"), false, time_emp))
        return false;
    if(!check_time(2, wxT("create_date"), false, time_emp))
        return false;
    if(!check_time(3, wxT("create_time"), false, time))
        return false;
    if(!check_time(4, wxT("create_timestamp"), false, time))
        return false;
    if(!check_time(5, wxT("ok"), false, time_emp))
        return false;
    if(!check_time(6, wxT("title"), false, time_emp))
        return false;
    if(!check_time(7, wxT("null_int"), true, time_emp))
        return false;

    logger.log_success("Test get_time");
    return true;
}


bool
check_date_time(int idx, const wxString &name, bool null, wxDateTime value){
    if(null){
        if(!check_null(idx, name, "Test get_date_time"))
            return false;
        value = wxDateTime();
    }
    if(testee.get_date_time(idx) != value){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT(" to be equal to ") << value.FormatISODate()
            << wxT(" ") << value.FormatISOTime()
            << wxT(" got ") << testee.get_date_time(idx).FormatISODate()
            << wxT(" ") << testee.get_date_time(idx).FormatISOTime();
        logger.log_error("Test get_date_time", msg.char_str());
        return false;
    }
    if(testee.get_date_time(name) != value){
        wxString msg = wxT("Expected column ");
        msg << name << wxT(" to be equal to ") << value.FormatISODate()
            << wxT(" ") << value.FormatISOTime()
            << wxT(" got ") << testee.get_date_time(name).FormatISODate()
            << wxT(" ") << testee.get_date_time(name).FormatISOTime();
        logger.log_error("Test get_date_time", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_get_date_time(){
    wxDateTime date, time, date_time, dt_emp;
    date.ParseDate(wxT("2000-01-01"));
    time.ParseTime(wxT("10:10"));
    date_time.ParseDateTime(wxT("2000-01-01 10:10"));

    if(!check_date_time(0, wxT("id"), false, dt_emp))
        return false;
    if(!check_date_time(1, wxT("fnum"), false, dt_emp))
        return false;
    if(!check_date_time(2, wxT("create_date"), false, date))
        return false;
    if(!check_date_time(3, wxT("create_time"), false, time))
        return false;
    if(!check_date_time(4, wxT("create_timestamp"), false, date_time))
        return false;
    if(!check_date_time(5, wxT("ok"), false, dt_emp))
        return false;
    if(!check_date_time(6, wxT("title"), false, dt_emp))
        return false;
    if(!check_date_time(7, wxT("null_int"), true, dt_emp))
        return false;

    logger.log_success("Test get_date_time");
    return true;
}


bool
check_bool(int idx, const wxString &name, bool null, bool value){
    if(null){
        if(!check_null(idx, name, "Test get_bool"))
            return false;
        value = false;
    }
    if(testee.get_bool(idx) != value){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT(" to be equal to ") << int(value)
            << wxT(" got ") << int(testee.get_bool(idx));
        logger.log_error("Test get_bool", msg.char_str());
        return false;
    }
    if(testee.get_bool(name) != value){
        wxString msg = wxT("Expected column ");
        msg << name << wxT(" to be equal to ") << int(value)
            << wxT(" got ") << int(testee.get_bool(name));
        logger.log_error("Test get_bool", msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_get_bool(){
    if(!check_bool(0, wxT("id"), false, true))
        return false;
    if(!check_bool(1, wxT("fnum"), false, true))
        return false;
    if(!check_bool(2, wxT("create_date"), false, false))
        return false;
    if(!check_bool(3, wxT("create_time"), false, false))
        return false;
    if(!check_bool(4, wxT("create_timestamp"), false, false))
        return false;
    if(!check_bool(5, wxT("ok"), false, true))
        return false;
    if(!check_bool(6, wxT("title"), false, true))
        return false;
    if(!check_bool(7, wxT("null_int"), true, false))
        return false;

    logger.log_success("Test get_bool");
    return true;
}


bool
run_tests(){
    testee.open(wxT("select * from data_row_test"));
    testee.fetch(0, wxPg::Query::FM_FORWARD_ALL);
    db.clean_sql_mon();
    bool result = true;
    if(test_cols)
        result = result && run_test_cols();
    if(test_types)
        result = result && run_test_types();
    if(test_get_int)
        result = result && run_test_get_int();
    if(test_get_float)
        result = result && run_test_get_float();
    if(test_get_str)
        result = result && run_test_get_str();
    if(test_get_date)
        result = result && run_test_get_date();
    if(test_get_time)
        result = result && run_test_get_time();
    if(test_get_date_time)
        result = result && run_test_get_date_time();
    if(test_get_bool)
        result = result && run_test_get_bool();
    testee.clear();
    return result;
}


int
main(int argc, char **argv){
    parse_options(argc, argv);
    if(!wxInitialize()){
        std::cerr << "Initialization of wxWidgets failed!";
        return 1;
    }
    if(!prepare_test_env())
        return 2;

    bool result = run_tests();

    drop_test_table();
    wxUninitialize();
    logger.output_summary();

    if(result)
        return 0;
    std::cerr << "Test completed with errors!";
    return 3;
}

