/* qb_test.cpp - test program for wxPg::Query_browser.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/



#include<iostream>
#include<fstream>
#include<cstdlib>
#include<ctime>
#include<ctime>
#include<wx/wx.h>
#include<wxPg/db/query_browser.h>
#include<getopt.h>
#include<coninfo.h>
#include<testdb.h>
#include<test_logger.h>


#define CHECK_TESTEE_STATE(idx, msg) \
if(!check_testee_state(idx, msg)){ \
    testee.clear(); \
    err_state_ = true; \
}


const int
TEST_BUFFER_SIZE_SMALL = 5,
TEST_BUFFER_SIZE_BIG = 50,
TEST_MAX_RECORD = 15;

int
test_buffer_size;

const wxChar
*TEST_SQL = wxT("select * from query_browser_test order by rownum"),
*TEST_FIELD = wxT("rownum");


wxPg::Query_browser
testee(&db);


bool
err_state_ = false;



void
print_help(){
    std::cout << "Test Query_browser.\n";
    std::cout << "Accepted options:\n";
    std::cout << "\t--help\t- Display help.\n";
    std::cout << "\t--test-fpnl\t- Test first(), prev(), next() and last().\n";
    std::cout << "\t--test-goto\t - Test go_to_record().\n";
    std::cout << "\t--test-fetch-first\t- Test fetch FM_FIRST.\n";
    std::cout << "\t--test-fetch-last\t- Test fetch FM_LAST.\n";
    std::cout << "\t--test-fetch-absolute\t- Test fetch FM_ABSOLUTE.\n";
    std::cout << "\t--test-fetch-relative\t- Test fetch FM_RELATIVE.\n";
    std::cout << "\t--test-fetch-forward\t- Test fetch FM_FORWARD.\n";
    std::cout << "\t--test-fetch-forward-all\t- Test fetch FM_FORWARD_ALL.\n";
    std::cout << "\t--test-fetch-backward\t- Test fetch FM_BACKWARD.\n";
    std::cout << "\t--test-fetch-backward-all\t- Test fetch FM_BACKWARD_ALL.\n";
    std::cout << "\t--test-move-first\t- Test move FM_FIRST.\n";
    std::cout << "\t--test-move-last\t- Test move FM_LAST.\n";
    std::cout << "\t--test-move-absolute\t- Test move FM_ABSOLUTE.\n";
    std::cout << "\t--test-move-relative\t- Test move FM_RELATIVE.\n";
    std::cout << "\t--test-move-forward\t- Test move FM_FORWARD.\n";
    std::cout << "\t--test-move-forward-all\t- Test move FM_FORWARD_ALL.\n";
    std::cout << "\t--test-move-backward\t- Test move FM_BACKWARD.\n";
    std::cout << "\t--test-move-backward-all\t- Test move FM_BACKWARD_ALL.\n";
}


int
test_fpnl,
test_goto,
test_fetch_first,
test_fetch_last,
test_fetch_absolute,
test_fetch_relative,
test_fetch_forward,
test_fetch_forward_all,
test_fetch_backward,
test_fetch_backward_all,
test_move_absolute,
test_move_relative,
test_move_first,
test_move_last,
test_move_forward,
test_move_forward_all,
test_move_backward,
test_move_backward_all;


option
options[] = {
    {"help", no_argument, NULL, 'h'},
    {"test-fpnl", no_argument, &test_fpnl, 1},
    {"test-goto", no_argument, &test_goto, 1},
    {"test-fetch-first", no_argument, &test_fetch_first, 1},
    {"test-fetch-last", no_argument, &test_fetch_last, 1},
    {"test-fetch-absolute", no_argument, &test_fetch_absolute, 1},
    {"test-fetch-relative", no_argument, &test_fetch_relative, 1},
    {"test-fetch-forward", no_argument, &test_fetch_forward, 1},
    {"test-fetch-forward-all", no_argument, &test_fetch_forward_all, 1},
    {"test-fetch-backward", no_argument, &test_fetch_backward, 1},
    {"test-fetch-backward-all", no_argument, &test_fetch_backward_all, 1},
    {"test-move-absolute", no_argument, &test_move_absolute, 1},
    {"test-move-relative", no_argument, &test_move_relative, 1},
    {"test-move-forward", no_argument, &test_move_forward, 1},
    {"test-move-forward-all", no_argument, &test_move_forward_all, 1},
    {"test-move-first", no_argument, &test_move_first, 1},
    {"test-move-last", no_argument, &test_move_last, 1},
    {"test-move-backward", no_argument, &test_move_backward, 1},
    {"test-move-backward-all", no_argument, &test_move_backward_all, 1},
    {NULL, 0, NULL, 0}
};


void
drop_test_table(){
    db.run(wxT("drop table query_browser_test"));
}


bool
prepare_test_env(int argc, char **argv){
    if(!db_connect())
        return false;

    int opt;
    while((opt = getopt_long(argc, argv, "", options, NULL)) != -1)
        if(opt == 'h'){
            print_help();
            exit(0);
        }

    if(!db.run(wxT("create table query_browser_test(rownum integer)"))){
        std::cerr << "Failed to create test table 'query_browser_test'!\n";
        std::cerr << db.last_err().char_str() << std::endl;
        exit(4);
    }
    wxString sql = wxT("insert into query_browser_test(rownum) ");
    sql += wxT("select * from generate_series(0, ");
    sql << TEST_MAX_RECORD;
    sql += wxT(")");
    if(!db.run(wxT("insert into query_browser_test(rownum) \
                    select * from generate_series(0, 15)"))){
        std::cerr << "Failed to insert test data!\n";
        std::cerr << db.last_err().char_str() << std::endl;
        exit(5);
    }

    if(test_fpnl == 0 && test_goto == 0 && test_fetch_first == 0
            && test_fetch_last == 0 && test_fetch_absolute == 0
            && test_fetch_relative == 0 && test_fetch_forward == 0
            && test_fetch_forward_all == 0 && test_fetch_backward_all == 0
            && test_fetch_backward == 0 && test_move_forward_all == 0
            && test_move_absolute == 0 && test_move_relative == 0
            && test_move_first == 0 && test_move_last == 0
            && test_move_forward == 0 && test_move_backward == 0
            && test_move_backward_all == 0)
        test_fpnl = test_goto = test_fetch_first = test_fetch_forward
        = test_fetch_forward_all = test_fetch_backward_all
        = test_fetch_absolute = test_fetch_relative = test_fetch_last
        = test_fetch_backward = test_move_forward_all = test_move_absolute
        = test_move_relative = test_move_first = test_move_last
        = test_move_forward = test_move_backward = test_move_backward_all = 1;

    std::srand(std::time(NULL));
    db.clean_sql_mon();

    return true;
}


bool
check_record_correctness(int value, const char *err_name){
    if(testee.end_reached() && testee.cursor_pos() > testee.record_count())
        return true;
    if(testee.get_int(TEST_FIELD) != value){
        wxString msg = wxT("at index ");
        msg << value << wxT(" wrong value [")
            << testee.get_int(TEST_FIELD) << wxT("]");
        logger.log_error(err_name, msg.char_str());
        return false;
    }else if(testee.abs_pos() != unsigned(value)){
        wxString msg;
        msg << wxT("expected abs_pos() ") << value
            << wxT(" got ") << testee.abs_pos();
        logger.log_error(err_name, msg.char_str());
        return false;
    }

    return true;
}


bool
check_index_correctness(const char *err_name){
    if(unsigned(testee.current()) > testee.abs_pos()){
        wxString msg = wxT("current() [");
        msg << testee.current() << wxT("] bigger than abs_pos() [")
            << testee.abs_pos() << wxT("].");
        logger.log_error(err_name, msg.char_str());
        return false;
    }else if(testee.abs_pos() > testee.cursor_pos()){
        wxString msg = wxT("abs_pos() [");
        msg << testee.abs_pos() << wxT("] bigger than cursor_pos() [")
            << testee.cursor_pos() << wxT("].");
        logger.log_error(err_name, msg.char_str());
        return false;
    }else if(testee.end_reached() && testee.record_count() != TEST_MAX_RECORD + 1){
        wxString msg = wxT("record count [");
        msg << testee.record_count() << wxT("] is incorrect [")
            << TEST_MAX_RECORD + 1 << wxT("]");
        logger.log_error(err_name, msg.char_str());
        return false;
    }else if(testee.cursor_pos() > testee.record_count() && !testee.end_reached()){
        wxString msg = wxT("cursor_pos() [");
        msg << testee.cursor_pos() << wxT("] bigger than record_count() [")
            << testee.record_count() << wxT("] with end_reached() == false.");
        logger.log_error(err_name, msg.char_str());
        return false;
    }else if(testee.cursor_pos() > testee.record_count() + 1){
        wxString msg = wxT("cursor_pos() [");
        msg << testee.cursor_pos() << wxT("] bigger than record_count() + 1 [")
            << (testee.record_count() + 1) << wxT("].");
        logger.log_error(err_name, msg.char_str());
        return false;
    }if(!testee.end_reached() && testee.cursor_pos() > testee.record_count()){
        wxString msg = wxT("cursor_pos() [");
        msg << testee.cursor_pos() << wxT("] bigger than record_count() [")
            << (testee.record_count() + 1) << wxT("], end not reached.");
        logger.log_error(err_name, msg.char_str());
        return false;
    }

    return true;
}


bool
check_testee_state(int index, const char *err_name){
    if(!check_record_correctness(index, err_name))
        return false;
    return check_index_correctness(err_name);
}


void
run_test_fpnl(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    CHECK_TESTEE_STATE(0, "open() error")
    if(err_state_)
        return;

    int i = 1;
    bool stop = false;
    do{
        stop = !testee.next();
        if(!stop)
            CHECK_TESTEE_STATE(i++, "next() error")
        if(err_state_)
            stop = true;
    }while(!stop);
    if(!err_state_ && testee.abs_pos() != TEST_MAX_RECORD)
        logger.log_error("next() error", "last record not reached");
    if(!err_state_)
        logger.log_success("next() part 1.");
    err_state_ = false;

    testee.first();
    CHECK_TESTEE_STATE(0, "first() error")
    if(!err_state_)
        logger.log_success("first() part 1.");
    err_state_ = false;

    stop = false;
    i = 1;
    do{
        stop = !testee.next();
        if(!stop)
            CHECK_TESTEE_STATE(i++, "next() error")
        if(err_state_)
            stop = true;
    }while(!stop);
    if(!err_state_)
        logger.log_success("next() part 2.");
    err_state_ = false;

    testee.clear();

    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();

    for(int i = 0; i < 2; ++i){
        testee.first();
        CHECK_TESTEE_STATE(0, "first() error")
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("first() part 2.");
    err_state_ = false;

    testee.last();
    CHECK_TESTEE_STATE(TEST_MAX_RECORD, "last() error")
    if(!err_state_)
        logger.log_success("last() part 1.");
    err_state_ = false;

    i = TEST_MAX_RECORD;
    do{
        stop = testee.prev();
        if(!stop)
            CHECK_TESTEE_STATE(--i, "prev() error")
        if(err_state_)
            stop = true;
    }while(!stop);
    if(!err_state_)
        logger.log_success("prev().");

    testee.last();
    CHECK_TESTEE_STATE(TEST_MAX_RECORD, "last() error")
    testee.last();
    CHECK_TESTEE_STATE(TEST_MAX_RECORD, "last() error")
    if(!err_state_)
        logger.log_success("last() part 2.");

    testee.clear();
}


void
run_test_goto(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(testee.go_to_record(TEST_MAX_RECORD + 1))
        logger.log_error("go_to_record() error", "went beyond successfully!");
    else logger.log_success("go_to_record() past last.");

    if(testee.go_to_record(0)){
        CHECK_TESTEE_STATE(0, "go to 0 error")
        if(!err_state_)
            logger.log_success("go_to_record(0)");
        err_state_ = false;
    }else logger.log_error("go_to_record() error", "go to 0 failed!");



    for(int i = 0; i < 100; ++i){
        int row = rand() % (TEST_MAX_RECORD + 1);
        int buf_pos = rand() % testee.buffer_size();
        unsigned long long abs_pos = testee.abs_pos();
        bool ok = testee.go_to_record(row, buf_pos);
        wxString msg = wxT("go to ");
        msg << row << wxT(" (") << buf_pos << wxT(")");
        if(row > TEST_MAX_RECORD && ok){
            wxString msg_sub = wxT("went beyond successfully: ");
            msg_sub += msg;
            logger.log_error("go_to_record() error", msg_sub.char_str());
            err_state_ = true;
        }else{
            if(!ok && abs_pos != testee.abs_pos()){
                wxString msg_sub = wxT("active record changed on error: was ");
                msg_sub << abs_pos << wxT(", now is ") << testee.abs_pos();
                msg_sub += wxT(" (");
                msg_sub += msg + wxT(")");
                logger.log_error("go_to_record() error", msg_sub.char_str());
                err_state_ = true;
            }
            CHECK_TESTEE_STATE(row, msg.char_str())
        }
        msg = wxT("go_to_record(): ") + msg;
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("go_to_record() random");
    err_state_ = false;

    testee.clear();
}


void
run_test_fetch_first(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    testee.next();
    db.clean_sql_mon();
    err_state_ = false;

    if(!testee.fetch(0, wxPg::Query::FM_FIRST)){
        logger.log_error("fetch(FM_FIRST)", "fetch failed.");
        err_state_ = true;
    }else if(testee.rows() != 1){
        logger.log_error("fetch(FM_FIRST)", "expected exactly 1 rows.");
        err_state_ = true;
    }else CHECK_TESTEE_STATE(0, "fetch(FM_FIRST)")
    if(!err_state_)
        logger.log_success("fetch(FM_FIRST)");
    err_state_ = false;

    testee.clear();
}


void
run_test_fetch_last(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    testee.next();
    db.clean_sql_mon();
    err_state_ = false;

    if(!testee.fetch(0, wxPg::Query::FM_LAST)){
        logger.log_error("fetch(FM_LAST)", "fetch failed.");
        err_state_ = true;
    }else if(testee.rows() != 1){
        logger.log_error("fetch(FM_LAST)", "expected exactly 1 rows.");
        err_state_ = true;
    }else CHECK_TESTEE_STATE(TEST_MAX_RECORD, "fetch(FM_LAST)")
    if(!err_state_)
        logger.log_success("fetch(FM_LAST)");
    err_state_ = false;

    testee.clear();
}


void
run_test_fetch_absolute(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(testee.fetch(0, wxPg::Query::FM_ABSOLUTE))
        logger.log_error("fetch(0, FM_ABSOLUTE)", "succeeded.");
    else logger.log_success("fetch(0, FM_ABSOLUTE)");

    if(test_buffer_size < TEST_MAX_RECORD){
        if(testee.fetch(TEST_MAX_RECORD + 2, wxPg::Query::FM_ABSOLUTE))
            logger.log_error("fetch(FM_ABSOLUTE)", "fetch beyond succeeded.");
        else logger.log_success("fetch(FM_ABSOLUTE) beyond");

        for(int i = test_buffer_size + 1; i <= TEST_MAX_RECORD; ++i){
            if(!testee.fetch(i, wxPg::Query::FM_ABSOLUTE)){
                logger.log_error("fetch(FM_ABSOLUTE)", "fetch failed.");
                err_state_ = true;
            }else if(testee.rows() != 1){
                logger.log_error("fetch(FM_ABSOLUTE)", "expected 1 row.");
                err_state_ = true;
            }else CHECK_TESTEE_STATE(i - 1, "fetch(FM_ABSOLUTE)")
            if(err_state_)
                break;
        }
        if(!err_state_)
            logger.log_success("fetch(FM_ABSOLUTE) in unknown area");
        err_state_ = false;
    }

    for(int i = 0; i < 100; ++i){
        int row = rand() % (TEST_MAX_RECORD + 1) + 1;
        if(!testee.fetch(row, wxPg::Query::FM_ABSOLUTE)){
            logger.log_error("fetch(FM_ABSOLUTE)", "fetch failed.");
            err_state_ = true;
        }else if(testee.rows() != 1){
            logger.log_error("fetch(FM_ABSOLUTE)", "expected 1 row.");
            err_state_ = true;
        }else CHECK_TESTEE_STATE(row - 1, "fetch(FM_ABSOLUTE)")
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("fetch(FM_ABSOLUTE) random");
    err_state_ = false;

    testee.clear();
}


void
run_test_fetch_relative_sub(int start, int diff, const char *name){
    int count = start;
    while(true){
        int pos = testee.cursor_pos();
        bool fail_expected = (pos + count > TEST_MAX_RECORD + 1)
                             || (pos + count < 1);
        bool fail = !testee.fetch(count, wxPg::Query::FM_RELATIVE);
        if(fail_expected){
            if(fail)
                break;
            logger.log_error("fetch(FM_RELATIVE)", "fail expected");
            err_state_ = true;
        }else if(testee.rows() != 1){
            logger.log_error("fetch(FM_RELATIVE)", "expected 1 row");
            err_state_ = true;
        }else CHECK_TESTEE_STATE(pos + count - 1, "fetch(FM_RELATIVE)")
        if(err_state_)
            break;
        count += diff;
    }
    if(!err_state_)
        logger.log_success(name);
    err_state_ = false;
}


void
run_test_fetch_relative(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    run_test_fetch_relative_sub(1, 1, "fetch(FM_RELATIVE) forward to unknown");
    run_test_fetch_relative_sub(-1, -1, "fetch(FM_RELATIVE) backward");
    run_test_fetch_relative_sub(1, 1, "fetch(FM_RELATIVE) forward to known");

    testee.clear();
}


void
run_test_fetch_forward(){
    const int FETCH_COUNT = 5;
    const int FETCH2_START = 5;

    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    int start_pos = testee.rows() - 1;
    testee.current(start_pos);
    CHECK_TESTEE_STATE(testee.current(), "open() or current(int) error")
    if(err_state_){
        testee.clear();
        return;
    }

    err_state_ = false;
    if(testee.fetch(FETCH_COUNT, wxPg::Query::FM_FORWARD)){
        ++start_pos;
        if(testee.rows() < FETCH_COUNT && !testee.end_reached()){
            logger.log_error("fetch(FM_FORWARD)", "Fetched less than requested, end not reached.");
            err_state_ = true;
        }else for(int i = 0; i < testee.rows(); ++i){
            testee.current(i);
            CHECK_TESTEE_STATE(start_pos + i, "fetch(FM_FORWARD)")
            if(err_state_)
                break;
        }
    }
    if(!err_state_)
        logger.log_success("fetch(FM_FORWARD) part 1.");
    err_state_ = false;

    if(!testee.move(FETCH2_START, wxPg::Query::FM_ABSOLUTE)){
        logger.log_error("move(FM_ABSOLUTE)", "failed in fetch-forward test.");
        err_state_ = true;
    }else{
        if(testee.fetch(FETCH_COUNT, wxPg::Query::FM_FORWARD)){
            start_pos = FETCH2_START;
            if(testee.rows() < FETCH_COUNT && !testee.end_reached()){
                logger.log_error("fetch(FM_FORWARD)", "Fetched less than requested, end not reached.");
                err_state_ = true;
            }else for(int i = 0; i < testee.rows(); ++i){
                testee.current(i);
                CHECK_TESTEE_STATE(start_pos + i, "fetch(FM_FORWARD)")
                if(err_state_)
                    break;
            }
        }
    }
    if(!err_state_)
        logger.log_success("fetch(FM_FORWARD) part 2.");
    err_state_ = false;

    testee.clear();
}


void
run_test_fetch_forward_all_sub(){
    int cursor_pos = testee.cursor_pos();
    if(!testee.fetch(0, wxPg::Query::FM_FORWARD_ALL)){
        logger.log_error("fetch(FM_FORWARD_ALL)", "fetch failed.");
        err_state_ = true;
    }else if(testee.rows() != TEST_MAX_RECORD - cursor_pos + 1){
        wxString msg = wxT("expected ");
        msg << (TEST_MAX_RECORD - cursor_pos + 1) << wxT(" rows, found ")
            << testee.rows();
        logger.log_error("fetch(FM_FORWARD_ALL)", msg.char_str());
        err_state_ = true;
    }else if(testee.rows() > 0)
        for(int i = cursor_pos; i <= TEST_MAX_RECORD; ++i){
        CHECK_TESTEE_STATE(i, "fetch(FM_FORWARD_ALL)")
        if(err_state_)
            break;
        else testee.current_move(1);
    }
}


void
run_test_fetch_forward_all(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(testee.cursor_pos() < TEST_MAX_RECORD){
        run_test_fetch_forward_all_sub();
        if(!err_state_)
            logger.log_success("fetch(FM_FORWARD_ALL) to unknown.");
        err_state_ = false;
    }

    for(int i = 0; i < 100; ++i){
        int start_pos = rand() % (TEST_MAX_RECORD + 1);
        int flag = (start_pos == 0) ? wxPg::Query::FM_BACKWARD_ALL : wxPg::Query::FM_ABSOLUTE;
        if(!testee.move(start_pos, flag)){
            logger.log_error("fetch(FM_FORWARD_ALL)", "move failed");
            err_state_ = true;
        }else run_test_fetch_forward_all_sub();
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("fetch(FM_FORWARD_ALL) random.");
    err_state_ = false;

    testee.clear();
}


void
run_test_fetch_backward_sub(int start_pos, int count){
    do{
        if(!testee.fetch(count, wxPg::Query::FM_BACKWARD)){
            logger.log_error("fetch(FM_BACKWARD)", "fetch failed.");
            err_state_ = true;
            break;
        }else if(testee.rows() < 0 || testee.rows() > count){
            wxString msg = wxT("expected ");
            msg << count << wxT(" rows, got ") << testee.rows() << wxT(".");
            logger.log_error("fetch(FM_BACKWARD)", msg.char_str());
            err_state_ = true;
            break;
        }else if(testee.rows() > 0)
            --start_pos;
            int check_pos = start_pos;
            do{
                CHECK_TESTEE_STATE(check_pos, "fetch(FM_BACKWARD)")
                if(err_state_)
                    break;
                if(testee.current() == 0)
                    break;
                testee.current_move(-1);
                --check_pos;
            }while(true);
        if(err_state_)
            break;
    }while(testee.rows() > 0);
}


void
run_test_fetch_backward_sub2(const char *msg, int move_count){
    if(!testee.move(0, wxPg::Query::FM_FORWARD_ALL)){
        logger.log_error("fetch(FM_BACKWARD)", "move to end failed.");
        err_state_ = true;
    }else{
        int start_pos = testee.cursor_pos() - 1;
        run_test_fetch_backward_sub(start_pos, move_count);
    }
    if(!err_state_)
        logger.log_success(msg);
    err_state_ = false;
}


void
run_test_fetch_backward(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    int start_pos = testee.cursor_pos() - 1;
    run_test_fetch_backward_sub(start_pos, 1);
    if(!err_state_)
        logger.log_success("fetch(FM_BACKWARD) part 1.");
    err_state_ = false;

    run_test_fetch_backward_sub2("fetch(FM_BACKWARD) part 2.", 4);
    run_test_fetch_backward_sub2("fetch(FM_BACKWARD) part 2.", 5);

    testee.clear();
}


void
run_test_fetch_backward_all_sub(){
    int start_pos = testee.cursor_pos();
    if(!testee.fetch(0, wxPg::Query::FM_BACKWARD_ALL)){
        logger.log_error("fetch(FM_BACKWARD_ALL)", "fetch failed.");
        err_state_ = true;
    }else if(testee.rows() != start_pos - 1){
        wxString msg = wxT("expected ");
        msg << (start_pos - 1) << wxT("rows, found ") << testee.rows();
        logger.log_error("fetch(FM_BACKWARD_ALL)", msg.char_str());
        err_state_ = true;
    }else if(testee.rows() > 0){
        for(int i = start_pos - 2; i >= 0; --i){
            CHECK_TESTEE_STATE(i, "fetch(FM_BACKWARD_ALL)")
            if(err_state_)
                break;
            testee.current_move(-1);
        }
    }
}


void
run_test_fetch_backward_all(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    run_test_fetch_backward_all_sub();
    if(!err_state_)
        logger.log_success("fetch(FM_BACKWARD_ALL) part 1.");
    err_state_ = false;

    for(int i = 0; i < 100; ++i){
        int start_pos = rand() % TEST_MAX_RECORD + 2;
        if(!testee.move(start_pos, wxPg::Query::FM_ABSOLUTE)){
            logger.log_error("fetch(FM_BACKWARD_ALL)", "move failed");
            err_state_ = true;
        }else run_test_fetch_backward_all_sub();
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("fetch(FM_BACKWARD_ALL) part 2.");
    err_state_ = false;

    testee.clear();
}


void
run_test_move_forward_all(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(!testee.move(0, wxPg::Query::FM_FORWARD_ALL))
        logger.log_error("move(FM_FORWARD_ALL)", "failed.");
    else if(testee.cursor_pos() != TEST_MAX_RECORD + 2){
        wxString msg = wxT("bad cursor pos: expected ");
        msg << (TEST_MAX_RECORD + 2) << wxT(" found ")
            << testee.cursor_pos() << wxT(".");
        logger.log_error("move(FM_FORWARD_ALL)", msg.char_str());
    }else if(!testee.end_reached())
        logger.log_error("move(FM_FORWARD_ALL)", "end not reached.");
    else if(testee.record_count() != TEST_MAX_RECORD + 1){
        wxString msg = wxT("bad record count: expected ");
        msg << (TEST_MAX_RECORD + 1) << wxT(" found ")
            << testee.record_count() << wxT(".");
        logger.log_error("move(FM_FORWARD_ALL)", msg.char_str());
    }else logger.log_success("move(FM_FORWARD_ALL)");

    testee.clear();
}


void
run_test_move_forward(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(test_buffer_size < TEST_MAX_RECORD){
        int cursor_pos = testee.cursor_pos();
        if(!testee.move(5, wxPg::Query::FM_FORWARD)){
            logger.log_error("move(FM_FORWARD)", "move failed.");
            err_state_ = true;
        }else if(testee.cursor_pos() != cursor_pos + 5){
            wxString msg = wxT("wrong cursor_pos(): expected 5, found ");
            msg << testee.cursor_pos() << wxT(".");
            logger.log_error("move(FM_FORWARD)", msg.char_str());
            err_state_ = true;
        }else{
            if(!testee.fetch(1)){
                logger.log_error("move(FM_FORWARD)", "fetch failed.");
                err_state_ = true;
            }
            CHECK_TESTEE_STATE(cursor_pos + 5, "move(FM_FORWARD)")
        }
    }else{
        int cursor_pos = testee.cursor_pos();
        if(testee.move(5, wxPg::Query::FM_FORWARD)){
            logger.log_error("move(FM_FORWARD)", "move beyond successfully.");
            err_state_ = true;
        }else if(testee.cursor_pos() > cursor_pos){
            wxString msg = wxT("wrong cursor_pos(): expected >=");
            msg << cursor_pos << wxT(", got ") << testee.cursor_pos();
            logger.log_error("move(FM_FORWARD)", msg.char_str());
            err_state_ = true;
        }
    }
    if(!err_state_)
        logger.log_success("move(FM_FORWARD) part 1.");
    err_state_ = false;

    for(int i = 0; i < 100; ++i){
        if(!testee.move(0, wxPg::Query::FM_BACKWARD_ALL)){
            logger.log_error("move(FM_FORWARD)", "failed to move backwards");
            err_state_ = true;
        }else{
            int rows = rand() % TEST_MAX_RECORD + 1;
            if(!testee.move(rows, wxPg::Query::FM_FORWARD)){
                logger.log_error("move(FM_FORWARD)", "move failed.");
                err_state_ = true;
            }else if(!testee.fetch(1)){
                logger.log_error("move(FM_FORWARD)", "fetch failed.");
                err_state_ = true;
            }else{
                if(rows == (TEST_MAX_RECORD + 1) && testee.rows() > 0){
                    logger.log_error("move(FM_FORWARD)", "fetched returned rows, when shouldn't");
                    err_state_ = true;
                }else CHECK_TESTEE_STATE(rows, "move(FM_FORWARD)")
            }
        }
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("move(FM_FORWARD) part 2.");
    err_state_ = false;

    testee.clear();
}


void
run_test_move_absolute(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(test_buffer_size < TEST_MAX_RECORD){
        bool ok = testee.move(test_buffer_size + 2, wxPg::Query::FM_ABSOLUTE);
        if(ok)
            ok = testee.fetch(1);
        if(ok){
            CHECK_TESTEE_STATE(test_buffer_size + 2, "move(FM_ABSOLUTE) part 1")
        }else{
            logger.log_error("move(FM_ABSOLUTE) part 1", "failed to move or fetch");
            err_state_ = true;
        }

        if(!err_state_)
            logger.log_success("move(FM_ABSOLUTE) part 1");
        err_state_ = false;
    }

    int row = testee.cursor_pos();
    if(testee.move(TEST_MAX_RECORD + 2, wxPg::Query::FM_ABSOLUTE)){
        logger.log_error("move(FM_ABSOLUTE) beyond", "didn't fail");
        err_state_ = true;
    }else if(!testee.fetch(1)){
        logger.log_error("move(FM_ABSOLUTE) beyond", "fetch failed");
        err_state_ = true;
    }else CHECK_TESTEE_STATE(row, "move(FM_ABSOLUTE) beyond")
    if(!err_state_)
        logger.log_success("move(FM_ABSOLUTE) beyond");
    err_state_ = false;

    for(int i = 0; i < 100; ++i){
        int pos = rand() % TEST_MAX_RECORD + 1;
        if(!testee.move(pos, wxPg::Query::FM_ABSOLUTE)){
            wxString msg = wxT("failed move(");
            msg << pos << wxT(")");
            logger.log_error("move(FM_ABSOLUTE)", msg.char_str());
            err_state_ = true;
        }else if(!testee.fetch(1)){
            logger.log_error("move(FM_ABSOLUTE)", "fetch failed");
            err_state_ = true;
        }else if(pos != TEST_MAX_RECORD)
            CHECK_TESTEE_STATE(pos, "move(FM_ABSOLUTE)")
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("move(FM_ABSOLUTE) random.");
    err_state_ = false;

    testee.clear();
}


void
run_test_move_relative(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(testee.move(-TEST_MAX_RECORD * 2, wxPg::Query::FM_RELATIVE))
        logger.log_error("move(FM_RELATIVE)", "move before start did't fail.");
    else logger.log_success("move(FM_RELATIVE) before start.");
    if(testee.move(TEST_MAX_RECORD * 2, wxPg::Query::FM_RELATIVE))
        logger.log_error("move(FM_RELATIVE)", "move beyond did't fail.");
    else logger.log_success("move(FM_RELATIVE) beyond.");

    for(int i = 0; i < 200; ++i){
        int positive = rand() % 2;
        int start_pos = testee.cursor_pos() - 1;
        int pos;
        if(positive)
            pos = rand() % (TEST_MAX_RECORD - testee.abs_pos() + 1);
        else pos = -1 * (rand() % (testee.abs_pos() + 1));
        if(pos == 0)
            pos = 1;

        if(!testee.move(pos, wxPg::Query::FM_RELATIVE)){
            continue;
        }else if(!testee.fetch(1)){
            logger.log_error("move(FM_RELATIVE)", "fetch failed");
            err_state_ = true;
        }else if(testee.rows())
            CHECK_TESTEE_STATE(start_pos + pos + 1, "move(FM_RELATIVE)")
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("move(FM_RELATIVE) random");
    err_state_ = false;

    testee.clear();
}


void
run_test_move_first(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(!testee.move(0, wxPg::Query::FM_FIRST)){
        logger.log_error("move(FM_FIRST)", "failed.");
        err_state_ = true;
    }else if(!testee.fetch(1)){
        logger.log_error("move(FM_FIRST)", "fetch failed.");
        err_state_ = true;
    }else CHECK_TESTEE_STATE(1, "move(FM_FIRST)")
    if(!err_state_)
        logger.log_success("move(FM_FIRST)");
    err_state_ = false;

    testee.clear();
}


void
run_test_move_last(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(!testee.move(0, wxPg::Query::FM_LAST)){
        logger.log_error("move(FM_LAST)", "failed.");
        err_state_ = true;
    }else if(!testee.fetch(1)){
        logger.log_error("move(FM_LAST)", "fetch failed.");
        err_state_ = true;
    }else{
        if(testee.rows()){
            logger.log_error("move(FM_LAST)", "fetched rows past end.");
            err_state_ = true;
        }else if(!testee.fetch(1, wxPg::Query::FM_BACKWARD)){
            logger.log_error("move(FM_LAST)", "fetch backward failed.");
            err_state_ = true;
        }else CHECK_TESTEE_STATE(TEST_MAX_RECORD, "move(FM_LAST)")
    }
    if(!err_state_)
        logger.log_success("move(FM_LAST)");
    err_state_ = false;

    testee.clear();
}


void
run_test_move_backward(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(test_buffer_size < TEST_MAX_RECORD){
        if(testee.move(test_buffer_size + 2, wxPg::Query::FM_BACKWARD))
            logger.log_error("move(FM_BACKWARD)", "fail expected.");
        else logger.log_success("move(FM_BACKWARD) part 1.");
    }

    if(!testee.move(5, wxPg::Query::FM_ABSOLUTE))
        logger.log_error("move(FM_BACKWARD)", "move absolute 5 failed.");
    else if(!testee.move(5, wxPg::Query::FM_BACKWARD))
        logger.log_error("move(FM_BACKWARD)", "move failed.");
    else if(testee.cursor_pos() != 0)
        logger.log_error("move(FM_BACKWARD)", "cursor_pos() != 0");
    else logger.log_success("move(FM_BACKWARD) part 2.");

    for(int i = 0; i < 100; ++i){
        if(!testee.move(0, wxPg::Query::FM_FORWARD_ALL)){
            logger.log_error("move(FM_BACKWARD)", "move to end failed.");
            err_state_ = true;
        }else{
            int count = rand() % (TEST_MAX_RECORD + 2) + 1;
            if(!testee.move(count, wxPg::Query::FM_BACKWARD)){
                logger.log_error("move(FM_BACKWARD)", "move failed.");
                err_state_ = true;
            }else if(count == TEST_MAX_RECORD + 2){
                if(testee.cursor_pos() != 0){
                    logger.log_error("move(FM_BACKWARD)", "cursor_pos() != 0");
                    err_state_ = true;
                }
            }else if(!testee.fetch(1)){
                logger.log_error("move(FM_BACKWARD)", "fetch failed.");
                err_state_ = true;
            }else CHECK_TESTEE_STATE(TEST_MAX_RECORD + 2 - count, "move(FM_BACKWARD)")
        }
        if(err_state_)
            break;
    }
    if(!err_state_)
        logger.log_success("move(FM_BACKWARD) part 3.");
    err_state_ = false;

    testee.clear();
}


void
run_test_move_backward_all(){
    if(!testee.open(TEST_SQL)){
        logger.log_error("SQL failure", "Main select failed!");
        return;
    }
    db.clean_sql_mon();
    err_state_ = false;

    if(!testee.move(0, wxPg::Query::FM_BACKWARD_ALL)){
        logger.log_error("move(FM_BACKWARD_ALL)", "move failed.");
        err_state_ = true;
    }else if(testee.cursor_pos() != 0){
        logger.log_error("move(FM_BACKWARD_ALL)", "cursor_pos() != 0.");
        err_state_ = true;
    }else if(!testee.fetch(1)){
        logger.log_error("move(FM_BACKWARD_ALL)", "fetch failed.");
        err_state_ = true;
    }else CHECK_TESTEE_STATE(0, "move(FM_BACKWARD_ALL)")
    if(!err_state_)
        logger.log_success("move(FM_BACKWARD_ALL)");
    err_state_ = false;

    testee.clear();
}


bool
run_tests_sub(){
    testee.buffer_size(test_buffer_size);
    err_state_ = false;
    if(test_fpnl)
        run_test_fpnl();
    err_state_ = false;
    if(test_goto)
        run_test_goto();
    err_state_ = false;
    if(test_fetch_first)
        run_test_fetch_first();
    if(test_fetch_last)
        run_test_fetch_last();
    if(test_fetch_absolute)
        run_test_fetch_absolute();
    if(test_fetch_relative)
        run_test_fetch_relative();
    if(test_fetch_forward)
        run_test_fetch_forward();
    if(test_fetch_forward_all)
        run_test_fetch_forward_all();
    if(test_fetch_backward)
        run_test_fetch_backward();
    if(test_fetch_backward_all)
        run_test_fetch_backward_all();
    if(test_move_forward)
        run_test_move_forward();
    if(test_move_forward_all)
        run_test_move_forward_all();
    if(test_move_absolute)
        run_test_move_absolute();
    if(test_move_relative)
        run_test_move_relative();
    if(test_move_first)
        run_test_move_first();
    if(test_move_last)
        run_test_move_last();
    if(test_move_backward)
        run_test_move_backward();
    if(test_move_backward_all)
        run_test_move_backward_all();
    return true;
}


bool
run_tests(){
    test_buffer_size = TEST_BUFFER_SIZE_SMALL;
    if(!run_tests_sub())
        return false;
    logger.log_check_point("Stage 1 completed!");
    test_buffer_size = TEST_BUFFER_SIZE_BIG;
    if(!run_tests_sub())
        return false;
    logger.log_check_point("Stage 2 completed!");
    return true;
}


int
main(int argc, char **argv){
    if(!wxInitialize()){
        std::cerr << "Initialization of wxWidgets failed!";
        return 1;
    }
    if(!prepare_test_env(argc, argv))
        return 2;

    if(!run_tests()){
        drop_test_table();
        return 3;
    }

    drop_test_table();
    wxUninitialize();
    logger.output_summary();
    return 0;
}
