/* test_logger.cpp - test_logger class for test programs.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "test_logger.h"
#include "testdb.h"
#include<iostream>


test_logger
logger;


test_logger::test_logger() : tests_(0), pass_(0){
    this->log_file_.open("test.log");
}


test_logger::~test_logger(){
    this->log_file_.flush();
    this->log_file_.close();
}


void
test_logger::log_success(const char *name){
    this->log_file_ << name <<  "\t[OK]\n";
    ++this->tests_;
    ++this->pass_;
    db.clean_sql_mon();
}


void
test_logger::log_error(const char *name, const char *detail){
    std::cerr << "Test failed: " << name << std::endl;
    this->log_file_ << name << "\t[FAILED]\n";
    this->log_file_ << "DETAILS:\n" << detail << std::endl;
    wxString sql = db.clean_sql_mon();
    this->log_file_ << sql.char_str() << std::endl;
    ++this->tests_;
}


void
test_logger::log_check_point(const char *name){
    this->log_file_ << name << std::endl;
}


void
test_logger::output_summary(){
    this->log_file_ << "Total tests: " << this->tests_
                    << "\tPassed: " << this->pass_
                    << "\tFailed: " << (this->tests_ - this->pass_) << std::endl;

    std::cout << "Total tests: " << this->tests_
              << "\tPassed: " << this->pass_
              << "\tFailed: " << (this->tests_ - this->pass_) << std::endl;
}
