/* updatable_row_test.cpp - test program for wxPg::Updatable_row.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<iostream>
#include<wx/wx.h>
#include<test_logger.h>
#include<testdb.h>
#include<wxPg/db/updatable_row.h>
#include<wxPg/utils/type.h>
#include<getopt.h>
#include<cstdlib>


int
test_setters_empty,
test_clear_empty,
test_type_empty,
test_load,
test_setters_loaded,
test_clear_loaded,
test_unload,
test_type_loaded,
test_insert,
test_update,
test_delete_record;


option
options[] = {
    {"help", no_argument, NULL, 'h'},
    {"test-setters-empty", no_argument, &test_setters_empty, 1},
    {"test-clear-empty", no_argument, &test_clear_empty, 1},
    {"test-type-empty", no_argument, &test_type_empty, 1},
    {"test-load", no_argument, &test_load, 1},
    {"test-setters-loaded", no_argument, &test_setters_loaded, 1},
    {"test-clear-loaded", no_argument, &test_clear_loaded, 1},
    {"test-unload", no_argument, &test_unload, 1},
    {"test-type-loaded", no_argument, &test_type_loaded, 1},
    {"test-insert", no_argument, &test_insert, 1},
    {"test-update", no_argument, &test_update, 1},
    {"test-delete_record", no_argument, &test_delete_record, 1},
    {NULL, 0, NULL, 0}
};


void
print_help(){
    std::cout << "Test program for wxPg::Updatable_row.\n"
              << "When no options are given, all tests are run.\n"
              << "Accepted options:\n"
              << "\t--help - display help and exit.\n"
              << "\t--test-setters-empty - test setters on empty row.\n"
              << "\t--test-clear-empty - test clear() on empty row.\n"
              << "\t--test-type-empty - test set_sype() on empty row.\n"
              << "\t--test-load - test load().\n"
              << "\t--test-setters-loaded - test setters on loaded row.\n"
              << "\t--test-clear-loaded - test clear() on loaded row.\n"
              << "\t--test-unload - test unload().\n"
              << "\t--test-type-loaded - test set_sype() on loaded row.\n"
              << "\t--test-insert - test insert().\n"
              << "\t--test-update - test update().\n"
              << "\t--test-delete_record - test delete_record().\n";
    std::exit(0);
}


void
parse_options(int argc, char **argv){
    int res;
    while((res = getopt_long(argc, argv, "", options, NULL)) >= 0)
        if(res == 'h')
            print_help();

    if(test_setters_empty == 0 && test_clear_empty == 0 && test_type_empty == 0
            && test_load == 0 && test_setters_loaded == 0 && test_clear_loaded == 0
            && test_unload == 0 && test_type_loaded == 0 && test_insert == 0
            && test_update == 0 && test_delete_record == 0)
        test_setters_empty = test_clear_empty = test_type_empty = test_load
        = test_setters_loaded = test_clear_loaded = test_unload
        = test_type_loaded = test_insert = test_update = test_delete_record = 1;
}


wxPg::Updatable_row
testee(&db);


void
drop_test_table(){
    db.run(wxT("drop table updatable_row_test"));
}


bool
prepare_test_env(){
    wxString create = wxT(
        "CREATE TABLE updatable_row_test( \
           id integer, \
           title text, \
           fnum double precision, \
           create_date date, \
           create_time time without time zone, \
           create_timestamp timestamp without time zone, \
           ok boolean \
        );");

    if(!db_connect())
        return false;
    if(!db.run(create)){
        logger.log_error("Fatal error", "Failed to create test table");
        return false;
    }
    if(!db.run(wxT("insert into updatable_row_test values(1, 'aaa', 2.5, \
                    '2005-05-05', '15:16:17', '2005-05-05 15:16:17', true)"))){
        logger.log_error("Fatal error", "Failed to insert data");
        drop_test_table();
        return false;
    }
    db.clean_sql_mon();

    return true;
}


bool
set_new_field_general_check(int col_count, bool must_fail,
                           const wxString &field, const char *msg){
    if(testee.cols() == col_count && !must_fail){
        logger.log_error(msg, "Unexpected fail.");
        return false;
    }
    if(testee.cols() == col_count + 1 && must_fail){
        logger.log_error(msg, "Fail expected.");
        return false;
    }
    if(testee.col_idx(field) < 0 && !must_fail){
        logger.log_error(msg, "New field does not exist.");
        return false;
    }

    return true;
}


bool
check_field_int(const wxString &field, int value, const char *test_name){
    if(testee.get_int(field) != value){
        wxString msg = wxT("Expected value ");
                 msg << value << wxT(" got ") << testee.get_int(field);
        logger.log_error(test_name, msg.char_str());
        return false;
    }

    return true;
}


bool
set_field_check(const wxString &field, int value, const char *msg){
    if(!check_field_int(field, value, msg))
        return false;
    if(testee.type_simple(field) != wxPg::TYPE_INT){
        logger.log_error(msg, "Data type not TYPE_INT.");
        return false;
    }

    return true;
}


bool
set_new_field_value(const wxString &field, int value, bool must_fail, const char *msg){
    int col_count = testee.cols();
    testee.set_value(field, value);
    if(!set_new_field_general_check(col_count, must_fail, field, msg))
        return false;
    if(must_fail)
        return true;
    return set_field_check(field, value, msg);
}


bool
check_field_float(const wxString &field, double value, const char *test_name){
    if(testee.get_float(field) != value){
        wxString msg = wxT("Expected value ");
        msg << value << wxT(" got ") << testee.get_float(field);
        logger.log_error(test_name, msg.char_str());
        return false;
    }

    return true;
}


bool
set_field_check(const wxString &field, double value, const char *msg){
    if(!check_field_float(field, value, msg))
        return false;
    if(testee.type_simple(field) != wxPg::TYPE_FLOAT){
        logger.log_error(msg, "Data type not TYPE_FLOAT.");
        return false;
    }

    return true;
}


bool
set_new_field_value(const wxString &field, double value, bool must_fail, const char *msg){
    int col_count = testee.cols();
    testee.set_value(field, value);
    if(!set_new_field_general_check(col_count, must_fail, field, msg))
        return false;
    if(must_fail)
        return true;
    return set_field_check(field, value, msg);
}


bool
check_field_str(const wxString &field, const wxString &value, const char *test_name){
    if(testee.get_str(field) != value){
        wxString msg = wxT("Expected value ");
        msg += value + wxT(" got ") + testee.get_str(field);
        logger.log_error(test_name, msg.char_str());
        return false;
    }

    return true;
}


bool
set_field_check(const wxString &field, const wxString &value, const char *msg){
    if(!check_field_str(field, value, msg))
        return false;
    if(testee.type_simple(field) != wxPg::TYPE_STRING){
        logger.log_error(msg, "Data type not TYPE_STRING.");
        return false;
    }

    return true;
}


bool
set_new_field_value(const wxString &field, const wxString &value,
                   bool must_fail, const char *msg){
    int col_count = testee.cols();
    testee.set_value(field, value);
    if(!set_new_field_general_check(col_count, must_fail, field, msg))
        return false;
    if(must_fail)
        return true;
    return set_field_check(field, value, msg);
}


bool
check_field_date(const wxString &field, const wxDateTime &value, const char *test_name){
    wxDateTime date = testee.get_date(field);
    if(!date.IsSameDate(value)){
        wxString message = wxT("Expected value ");
        message << value.FormatISODate() << wxT(" got ")
                << date.FormatISODate();
        logger.log_error(test_name, message.char_str());
        return false;
    }

    return true;
}


bool
set_field_check_d(const wxString &field, const wxDateTime &value, const char *msg){
    if(!check_field_date(field, value, msg))
        return false;
    if(testee.type_simple(field) != wxPg::TYPE_DATE){
        logger.log_error(msg, "Data type not TYPE_DATE.");
        return false;
    }

    return true;
}


bool
set_new_field_value_d(const wxString &field, const wxDateTime &value,
                     bool must_fail, const char *msg){
    int col_count = testee.cols();
    testee.set_date(field, value);
    if(!set_new_field_general_check(col_count, must_fail, field, msg))
        return false;
    if(must_fail)
        return true;
    return set_field_check_d(field, value, msg);
}


bool
check_field_time(const wxString &field, const wxDateTime &value, const char *test_name){
    wxDateTime date = testee.get_time(field);
    if(!date.IsSameTime(value)){
        wxString message = wxT("Expected value ");
        message << value.FormatISOTime() << wxT(" got ")
                << date.FormatISOTime();
        logger.log_error(test_name, message.char_str());
        return false;
    }

    return true;
}


bool
set_field_check_t(const wxString &field, const wxDateTime &value, const char *msg){
    if(!check_field_time(field, value, msg))
        return false;
    if(testee.type_simple(field) != wxPg::TYPE_TIME){
        logger.log_error(msg, "Data type not TYPE_TIME.");
        return false;
    }

    return true;
}


bool
set_new_field_value_t(const wxString &field, const wxDateTime &value,
                     bool must_fail, const char *msg){
    int col_count = testee.cols();
    testee.set_time(field, value);
    if(!set_new_field_general_check(col_count, must_fail, field, msg))
        return false;
    if(must_fail)
        return true;
    return set_field_check_t(field, value, msg);
}


bool
check_field_timestamp(const wxString &field, const wxDateTime &value, const char *test_name){
    wxDateTime date = testee.get_date_time(field);
    if(!date.IsEqualTo(value)){
        wxString message = wxT("Expected value ");
        message << value.Format(wxT("%Y-%m-%d %H:%M:%S")) << wxT(" got ")
                << date.Format(wxT("%Y-%m-%d %H:%M:%S"));
        logger.log_error(test_name, message.char_str());
        return false;
    }

    return true;
}


bool
set_field_check(const wxString &field, const wxDateTime &value, const char *msg){
    wxDateTime date = testee.get_date_time(field);
    if(!check_field_timestamp(field, date, msg))
        return false;
    if(testee.type_simple(field) != wxPg::TYPE_TIMESTAMP){
        logger.log_error(msg, "Data type not TYPE_TIMESTAMP.");
        return false;
    }

    return true;
}


bool
set_new_field_value(const wxString &field, const wxDateTime &value,
                   bool must_fail, const char *msg){
    int col_count = testee.cols();
    testee.set_value(field, value);
    if(!set_new_field_general_check(col_count, must_fail, field, msg))
        return false;
    if(must_fail)
        return true;
    return set_field_check(field, value, msg);
}


bool
check_field_bool(const wxString &field, bool value, const char *test_name){
    if(testee.get_bool(field) != value){
        wxString msg = wxT("Expected value ");
        if(value)
            msg += wxT("true");
        else msg += wxT("false");
        msg += wxT(" got ");
        if(testee.get_bool(field))
            msg += wxT("true");
        else msg += wxT("false");
        logger.log_error(test_name, msg.char_str());
        return false;
    }

    return true;
}


bool
set_field_check(const wxString &field, bool value, const char *msg){
    if(!check_field_bool(field, value, msg))
        return false;
    if(testee.type_simple(field) != wxPg::TYPE_BOOL){
        logger.log_error(msg, "Data type not TYPE_BOOL.");
        return false;
    }

    return true;
}


bool
set_new_field_value(const wxString &field, bool value,
                   bool must_fail, const char *msg){
    int col_count = testee.cols();
    testee.set_bool(field, value);
    if(!set_new_field_general_check(col_count, must_fail, field, msg))
        return false;
    if(must_fail)
        return true;
    return set_field_check(field, value, msg);
}


bool
set_field_check_null(const wxString &field, const char *msg){
    if(!testee.null(field)){
        logger.log_error(msg, "Field not set to NULL.");
        return false;
    }
    if(testee.type_simple(field) != wxPg::TYPE_UNKNOWN){
        logger.log_error(msg, "Data type not TYPE_UNKNOWN.");
        return false;
    }

    return true;
}


bool
set_new_field_null(const wxString &field, bool must_fail, const char *msg){
    int col_count = testee.cols();
    testee.set_null(field);
    if(!set_new_field_general_check(col_count, must_fail, field, msg))
        return false;
    return set_field_check_null(field, msg);
}


bool
test_set_field_value(const wxString &field, const char *msg){
    testee.set_value(field, 5);
    if(!set_field_check(field, 5, msg))
        return false;
    testee.set_value(field, 6.5);
    if(!set_field_check(field, 6.5, msg))
        return false;
    testee.set_value(field, wxT("aaa"));
    if(!set_field_check(field, wxString(wxT("aaa")), msg))
        return false;
    wxDateTime dt;
    dt.ParseDate(wxT("2009.05.05"));
    testee.set_date(field, dt);
    if(!set_field_check_d(field, dt, msg))
        return false;
    dt.ParseTime(wxT("12:15:17"));
    testee.set_time(field, dt);
    if(!set_field_check_t(field, dt, msg))
        return false;
    dt.ParseDateTime(wxT("2009.05.05 12:15:17"));
    testee.set_value(field, dt);
    if(!set_field_check(field, dt, msg))
        return false;
    testee.set_bool(field, true);
    if(!set_field_check(field, true, msg))
        return false;
    testee.set_null(field);
    if(!set_field_check_null(field, msg))
        return false;

    return true;
}


bool
run_test_setters_empty(){
    testee.unload();
    if(testee.loaded() || testee.cols()){
        logger.log_error("Fatal error", "Failed to prepare clean testee.");
        return false;
    }

    bool result = set_new_field_value(wxT("int_field"), 1, false, "Set int empty");
    if(result)
        logger.log_success("Set int empty");
    else return false;

    result = set_new_field_value(wxT("float_field"), 2.5, false, "Set float empty");
    if(result)
        logger.log_success("Set float empty");
    else return false;

    result = set_new_field_value(wxT("string_field"), wxString(wxT("aaa")), false, "Set string empty");
    if(result)
        logger.log_success("Set string empty");
    else return false;

    wxDateTime dt;
    dt.ParseDate(wxT("2009.05.05"));
    result = set_new_field_value_d(wxT("date_field"), dt, false, "Set date empty");
    if(result)
        logger.log_success("Set date empty");
    else return false;

    dt.ParseTime(wxT("12:15:17"));
    result = set_new_field_value_t(wxT("time_field"), dt, false, "Set time empty");
    if(result)
        logger.log_success("Set date empty");
    else return false;

    dt.ParseDateTime(wxT("2009.05.05 12:15:17"));
    result = set_new_field_value(wxT("timestamp_field"), dt, false, "Set date&time empty");
    if(result)
        logger.log_success("Set date&time empty");
    else return false;

    result = set_new_field_value(wxT("bool_field"), true, false, "Set bool empty");
    if(result)
        logger.log_success("Set bool empty");
    else return false;

    result = set_new_field_null(wxT("null_field"), false, "Set NULL empty");
    if(result)
        logger.log_success("Set NULL empty");
    else return false;

    result = test_set_field_value(wxT("bool_field"), "Set existing field");
    if(result)
        logger.log_success("Set existing field");
    else return false;

    return true;
}


bool
check_field_is_removed(const wxString &field, const char *title){
    if(testee.col_idx(field) >= 0)
    {
        wxString msg = wxT("Not removed column: ");
        msg << field;
        logger.log_error(title, msg.char_str());
        return false;
    }

    return true;
}


bool
run_test_clear_empty(){
    testee.set_null(wxT("field1"));
    testee.set_null(wxT("field2"));
    testee.set_null(wxT("field3"));
    if(testee.cols() == 0)
    {
        logger.log_error("Fatal error", "set_null() deasn set fields on empty.");
        return false;
    }
    testee.clear();
    if(testee.cols() != 0){
        logger.log_error("Clear empty", "Not cleared columns.");
        return false;
    }
    if(!check_field_is_removed(wxT("field1"), "Clear empty"))
        return false;
    if(!check_field_is_removed(wxT("field2"), "Clear empty"))
        return false;
    if(!check_field_is_removed(wxT("field3"), "Clear empty"))
        return false;

    logger.log_success("Clear empty");
    return true;
}


#define CHANGE_TYPE_NULL(type) \
    testee.set_type_simple(wxT("type_field"), wxPg::type); \
    if(testee.type_simple(wxT("type_field")) != wxPg::type){ \
        logger.log_error("Test type empty NULL", "Type not set to " # type); \
        return false; \
    } \
    if(!testee.null(wxT("type_field"))){ \
        logger.log_error("Test type empty NULL", "Not NULL after " # type); \
        return false; \
    }


bool
test_type_empty_NULL(){
    testee.set_null(wxT("type_field"));
    if(testee.type_simple(wxT("type_field")) != wxPg::TYPE_UNKNOWN){
        logger.log_error("Test type empty NULL", "Failed to prepare for NULL test");
        return false;
    }

    CHANGE_TYPE_NULL(TYPE_BOOL)
    CHANGE_TYPE_NULL(TYPE_INT)
    CHANGE_TYPE_NULL(TYPE_FLOAT)
    CHANGE_TYPE_NULL(TYPE_STRING)
    CHANGE_TYPE_NULL(TYPE_DATE)
    CHANGE_TYPE_NULL(TYPE_TIME)
    CHANGE_TYPE_NULL(TYPE_TIMESTAMP)

    logger.log_success("Test type empty NULL");
    return true;
}


bool
set_field_int(const wxString &field, const char *test_name){
    testee.set_value(field, 5);
    if(!check_field_int(field, 5, test_name))
        return false;

    return true;
}


#define CHANGE_TYPE(type) \
    testee.set_type_simple(field, wxPg::type); \
    if(testee.type_simple(field) != wxPg::type){ \
        logger.log_error(test_name, "type not changed to " # type); \
        return false; \
    }



#define CHECK_FOR_NULL() \
    if(!testee.null(field)){ \
        logger.log_error(test_name, "expected value: NULL"); \
        return false; \
    }


bool
test_int_to_else(const wxString &field, const char *test_name){
    if(!set_field_int(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_BOOL)
    if(!check_field_bool(field, true, test_name))
        return false;

    if(!set_field_int(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_FLOAT)
    if(!check_field_float(field, 5.0, test_name))
        return false;

    if(!set_field_int(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_STRING)
    if(!check_field_str(field, wxT("5"), test_name))
        return false;

    if(!set_field_int(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_DATE)
    CHECK_FOR_NULL()

    if(!set_field_int(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIME)
    CHECK_FOR_NULL()

    if(!set_field_int(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIMESTAMP)
    CHECK_FOR_NULL()

    logger.log_success(test_name);
    return true;
}


bool
set_field_float(const wxString &field, const char *test_name){
    testee.set_value(field, 3.5);
    if(!check_field_float(field, 3.5, test_name))
        return false;

    return true;
}


bool
test_float_to_else(const wxString &field, const char *test_name){
    if(!set_field_float(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_BOOL)
    if(!check_field_bool(field, true, test_name))
        return false;

    if(!set_field_float(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_INT)
    if(!check_field_int(field, 3, test_name))
        return false;

    if(!set_field_float(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_STRING)
    if(!check_field_str(field, wxT("3.5"), test_name))
        return false;

    if(!set_field_float(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_DATE)
    CHECK_FOR_NULL()

    if(!set_field_float(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIME)
    CHECK_FOR_NULL()

    if(!set_field_float(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIMESTAMP)
    CHECK_FOR_NULL()

    logger.log_success(test_name);
    return true;
}


bool
set_field_str(const wxString &field, const char *test_name, const wxString &val){
    testee.set_value(field, val);
    if(!check_field_str(field, val, test_name))
        return false;

    return true;
}


bool
set_field_str_num(const wxString &field, const char *test_name){
    return set_field_str(field, test_name, wxT("3.5"));
}


bool
set_field_str_dt(const wxString &field, const char *test_name){
    return set_field_str(field, test_name, wxT("2009-05-05 15:16:17"));
}


bool
set_field_string(const wxString &field, const char *test_name){
    return set_field_str(field, test_name, wxT("aaa"));
}


bool
test_str_to_else(const wxString &field, const char *test_name){
    if(!set_field_str_num(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_BOOL)
    if(!check_field_bool(field, true, test_name))
        return false;

    if(!set_field_str_dt(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_BOOL)
    if(!check_field_bool(field, true, test_name))
        return false;

    if(!set_field_string(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_BOOL)
    if(!check_field_bool(field, true, test_name))
        return false;



    if(!set_field_str_num(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_INT)
    if(!check_field_int(field, 3, test_name))
        return false;

    if(!set_field_str_dt(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_INT)
    if(!check_field_int(field, 2009, test_name))
        return false;

    if(!set_field_string(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_INT)
    CHECK_FOR_NULL()


    if(!set_field_str_num(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_FLOAT)
    if(!check_field_float(field, 3.5, test_name))
        return false;

    if(!set_field_str_dt(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_FLOAT)
    if(!check_field_float(field, 2009.0, test_name))
        return false;

    if(!set_field_string(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_FLOAT)
    CHECK_FOR_NULL()

    if(!set_field_str_num(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_DATE)
    CHECK_FOR_NULL()

    wxDateTime dt;
    dt.ParseDateTime(wxT("2009-05-05 15:16:17"));

    if(!set_field_str_dt(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_DATE)
    if(!check_field_date(field, dt, test_name))
        return false;

    if(!set_field_string(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_DATE)
    CHECK_FOR_NULL()


    if(!set_field_str_num(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIME)
    CHECK_FOR_NULL()

    if(!set_field_str_dt(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIME)
    if(!check_field_time(field, dt, test_name))
        return false;

    if(!set_field_string(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIME)
    CHECK_FOR_NULL()


    if(!set_field_str_num(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIMESTAMP)
    CHECK_FOR_NULL()

    if(!set_field_str_dt(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIMESTAMP)
    if(!check_field_timestamp(field, dt, test_name))
        return false;

    if(!set_field_string(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIMESTAMP)
    CHECK_FOR_NULL()

    logger.log_success(test_name);
    return true;
}


bool
set_field_date(const wxString &field, const char *test_name){
    wxDateTime date;
    date.ParseDate(wxT("2009-05-05"));
    testee.set_date(field, date);
    if(!check_field_date(field, date, test_name))
        return false;

    return true;
}


bool
test_date_to_else(const wxString &field, const char *test_name){
    if(!set_field_date(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_INT)
    if(!check_field_int(field, 2009, test_name))
        return false;

    if(!set_field_date(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_FLOAT)
    if(!check_field_float(field, 2009.0, test_name))
        return false;

    if(!set_field_date(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_STRING)
    if(!check_field_str(field, wxT("2009-05-05"), test_name))
        return false;

    if(!set_field_date(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIME)
    CHECK_FOR_NULL()

    if(!set_field_date(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIMESTAMP)
    wxDateTime dt;
    dt.ParseDate(wxT("2009-05-05"));
    if(!check_field_timestamp(field, dt, test_name))
        return false;

    if(!set_field_date(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_BOOL)
    if(!check_field_bool(field, false, test_name))
        return false;

    logger.log_success(test_name);
    return true;
}


bool
set_field_time(const wxString &field, const char *test_name){
    wxDateTime time;
    time.ParseTime(wxT("15:16:17"));
    testee.set_time(field, time);
    if(!check_field_time(field, time, test_name))
        return false;

    return true;
}


bool
test_time_to_else(const wxString &field, const char *test_name){
    if(!set_field_time(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_INT)
    if(!check_field_int(field, 15, test_name))
        return false;

    if(!set_field_time(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_FLOAT)
    if(!check_field_float(field, 15.0, test_name))
        return false;

    if(!set_field_time(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_STRING)
    if(!check_field_str(field, wxT("15:16:17"), test_name))
        return false;

    if(!set_field_time(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_DATE)
    CHECK_FOR_NULL()

    if(!set_field_time(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIMESTAMP)
    wxDateTime time;
    time.ParseTime(wxT("15:16:17"));
    if(!check_field_time(field, time, test_name))
        return false;

    if(!set_field_time(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_BOOL)
    if(!check_field_bool(field, false, test_name))
        return false;

    logger.log_success(test_name);
    return true;
}


bool
set_field_timestamp(const wxString &field, const char *test_name){
    wxDateTime dt;
    dt.ParseDateTime(wxT("2009-05-05 15:16:17"));
    testee.set_value(field, dt);
    if(!check_field_timestamp(field, dt, test_name))
        return false;

    return true;
}


bool
test_timestamp_to_else(const wxString &field, const char *test_name){
    if(!set_field_timestamp(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_INT)
    if(!check_field_int(field, 2009, test_name))
        return false;

    if(!set_field_timestamp(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_FLOAT)
    if(!check_field_float(field, 2009.0, test_name))
        return false;

    if(!set_field_timestamp(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_STRING)
    if(!check_field_str(field, wxT("2009-05-05 15:16:17"), test_name))
        return false;

    if(!set_field_timestamp(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_DATE)
    wxDateTime dt;
    dt.ParseDateTime(wxT("2009-05-05 15:16:17"));
    if(!check_field_date(field, dt, test_name))
        return false;

    if(!set_field_timestamp(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIME)
    if(!check_field_time(field, dt, test_name))
        return false;

    if(!set_field_timestamp(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_BOOL)
    if(!check_field_bool(field, false, test_name))
        return false;

    logger.log_success(test_name);
    return true;
}


bool
set_field_bool(const wxString &field, const char *test_name){
    testee.set_bool(field, true);
    if(!check_field_bool(field, true, test_name))
        return false;

    return true;
}


bool
test_bool_to_else(const wxString &field, const char *test_name){
    if(!set_field_bool(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_INT)
    if(!check_field_int(field, 1, test_name))
        return false;

    if(!set_field_bool(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_FLOAT)
    if(!check_field_float(field, 1.0, test_name))
        return false;

    if(!set_field_bool(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_STRING)
    if(!check_field_str(field, wxT("t"), test_name))
        return false;

    if(!set_field_bool(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_DATE)
    CHECK_FOR_NULL()

    if(!set_field_bool(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIME)
    CHECK_FOR_NULL()

    if(!set_field_bool(field, test_name))
        return false;
    CHANGE_TYPE(TYPE_TIMESTAMP)
    CHECK_FOR_NULL()

    logger.log_success(test_name);
    return true;
}


bool
run_test_type_empty(){
    bool result = test_type_empty_NULL();
    result = result && test_int_to_else(wxT("int_field"), "int to else");
    result = result && test_float_to_else(wxT("float_field"), "float to else");
    result = result && test_str_to_else(wxT("string_field"), "string to else");
    result = result && test_date_to_else(wxT("date_field"), "date to else");
    result = result && test_time_to_else(wxT("time_field"), "time to else");
    result = result && test_timestamp_to_else(wxT("timestamp_field"), "timestamp to else");
    result = result && test_bool_to_else(wxT("bool_field"), "bool to else");
    return result;
}


bool
check_col_name(const wxString &field, int idx, const char *test_name){
    if(testee.col_name(idx) != field){
        wxString msg = wxT("Expected column ");
        msg << idx << wxT("to be ") << field << wxT(" got ")
            << testee.col_name(idx);
        logger.log_error(test_name, msg.char_str());
        return false;
    }

    return true;
}


bool
check_field_null(const wxString &field, const char *test_name){
    if(!testee.null(field)){
        wxString msg = wxT("Expected ") + field + wxT(" to be NULL.");
        logger.log_error(test_name, msg.char_str());
        return false;
    }

    return true;
}


bool check_std_cols(const char *test_name){
    if(testee.cols() != 7){
        wxString msg = wxT("Expecte 7 columns, got ");
        msg << testee.cols();
        logger.log_error(test_name, msg.char_str());
        return false;
    }

    if(!check_col_name(wxT("id"), 0, test_name))
        return false;
    if(!check_col_name(wxT("title"), 1, test_name))
        return false;
    if(!check_col_name(wxT("fnum"), 2, test_name))
        return false;
    if(!check_col_name(wxT("create_date"), 3, test_name))
        return false;
    if(!check_col_name(wxT("create_time"), 4, test_name))
        return false;
    if(!check_col_name(wxT("create_timestamp"), 5, test_name))
        return false;
    if(!check_col_name(wxT("ok"), 6, test_name))
        return false;

    return true;
}


bool
check_std_fields_null(const char *test_name){
    if(!check_field_null(wxT("id"), test_name))
        return false;
    if(!check_field_null(wxT("title"), test_name))
        return false;
    if(!check_field_null(wxT("fnum"), test_name))
        return false;
    if(!check_field_null(wxT("create_date"), test_name))
        return false;
    if(!check_field_null(wxT("create_time"), test_name))
        return false;
    if(!check_field_null(wxT("create_timestamp"), test_name))
        return false;
    if(!check_field_null(wxT("ok"), test_name))
        return false;

    return true;
}


bool
check_std_values(const char *test_name){
    if(!check_field_int(wxT("id"), 1, test_name))
        return false;
    if(!check_field_str(wxT("title"), wxT("aaa"), test_name))
        return false;
    if(!check_field_float(wxT("fnum"), 2.5, test_name))
        return false;
    wxDateTime dt;
    dt.ParseDateTime(wxT("2005-05-05 15:16:17"));
    if(!check_field_date(wxT("create_date"), dt, test_name))
        return false;
    if(!check_field_time(wxT("create_time"), dt, test_name))
        return false;
    if(!check_field_timestamp(wxT("create_timestamp"), dt, test_name))
        return false;
    if(!check_field_bool(wxT("ok"), true, test_name))
        return false;

    return true;
}


bool
run_test_load(){
    testee.table(wxT("updatable_row_test"));
    if(!testee.load()){
        wxString msg = wxT("Failed with no condition: ");
        msg += testee.last_err();
        logger.log_error("load no cond", msg.char_str());
        return false;
    }
    if(!check_std_cols("load no cond"))
        return false;
    if(!check_std_fields_null("load no cond"))
        return false;
    logger.log_success("load no cond");

    if(!testee.load(wxT("id = 1"))){
        wxString msg = wxT("Failed with condition: ");
        msg += testee.last_err();
        logger.log_error("load with cond", msg.char_str());
        return false;
    }
    if(!check_std_cols("load with cond"))
        return false;
    if(!check_std_values("load with cond"))
        return false;

    logger.log_success("load with cond");
    return true;
}


bool
run_test_setters_loaded(){
    testee.unload();
    if(testee.loaded() || testee.cols()){
        logger.log_error("Fatal error", "Failed to prepare clean testee.");
        return false;
    }
    testee.table(wxT("updatable_row_test"));
    if(!testee.load()){
        wxString msg = wxT("failed to load.") + testee.last_err();
        logger.log_error("Fatal error", msg.char_str());
        return false;
    }

    bool result = set_new_field_value(wxT("int_field"), 1, true, "Set int non-existent");
    if(result)
        logger.log_success("Set int non-existent");
    else return false;

    result = set_new_field_value(wxT("float_field"), 2.5, true, "Set float non-existent");
    if(result)
        logger.log_success("Set float non-existent");
    else return false;

    result = set_new_field_value(wxT("string_field"), wxString(wxT("aaa")),
                                 true, "Set string non-existent");
    if(result)
        logger.log_success("Set string non-existent");
    else return false;

    wxDateTime dt;
    dt.ParseDate(wxT("2009.05.05"));
    result = set_new_field_value_d(wxT("date_field"), dt, true, "Set date non-existent");
    if(result)
        logger.log_success("Set date non-existent");
    else return false;

    dt.ParseTime(wxT("12:15:17"));
    result = set_new_field_value_t(wxT("time_field"), dt, true, "Set time non-existent");
    if(result)
        logger.log_success("Set date non-existent");
    else return false;

    dt.ParseDateTime(wxT("2009.05.05 12:15:17"));
    result = set_new_field_value(wxT("timestamp_field"), dt, true, "Set date&time non-existent");
    if(result)
        logger.log_success("Set date&time non-existent");
    else return false;

    result = set_new_field_value(wxT("bool_field"), true, true, "Set bool non-existent");
    if(result)
        logger.log_success("Set bool non-existent");
    else return false;

    result = set_new_field_null(wxT("null_field"), true, "Set NULL non-existent");
    if(result)
        logger.log_success("Set NULL non-existent");
    else return false;

    result = test_set_field_value(wxT("ok"), "Set existing field loaded");
    if(result)
        logger.log_success("Set existing field loaded");
    else return false;

    return true;
}


bool
run_test_clear_loaded(){
    testee.unload();
    if(testee.loaded() || testee.cols()){
        logger.log_error("Fatal error", "Failed to prepare clean testee.");
        return false;
    }
    testee.table(wxT("updatable_row_test"));
    if(!testee.load()){
        wxString msg = wxT("failed to load.") + testee.last_err();
        logger.log_error("Fatal error", msg.char_str());
        return false;
    }

    int cols = testee.cols();
    if(cols == 0){
        logger.log_error("Fatal error", "unexpected to load 0 columns.");
        return false;
    }

    testee.clear();
    if(testee.cols() != cols){
        wxString msg = wxT("Expected ");
        msg << cols << wxT(" columns, got ") << testee.cols();
        logger.log_error("clear loaded", msg.char_str());
        return false;
    }
    if(!check_std_fields_null("clear loaded"))
        return false;

    logger.log_success("clear loaded");
    return true;
}


bool
run_test_unload(){
    testee.unload();
    if(testee.loaded() || testee.cols()){
        logger.log_error("Fatal error", "Failed to prepare clean testee.");
        return false;
    }
    testee.table(wxT("updatable_row_test"));
    if(!testee.load()){
        wxString msg = wxT("failed to load.") + testee.last_err();
        logger.log_error("Fatal error", msg.char_str());
        return false;
    }

    testee.unload();
    if(testee.cols() != 0){
        logger.log_error("unload", "expected 0 columns");
        return false;
    }

    logger.log_success("unload");
    return true;
}


bool
change_type_loaded(const wxString &field, int type, const char *test_name){
    wxString value = testee.get_str(field);
    bool null_state = testee.null(field);
    int type_orig = testee.type_simple(field);
    testee.set_type_simple(field, type);
    if(testee.type_simple(field) != type_orig){
        logger.log_error(test_name, "type changed");
        return false;
    }
    if(testee.get_str(field) != value){
        logger.log_error(test_name, "value changed");
        return false;
    }
    if(testee.null(field) != null_state){
        logger.log_error(test_name, "NULL state changed");
        return false;
    }

    return true;
}


bool
change_field_type(const wxString &field, const char *test_name){
    int type = testee.type_simple(field);
    if(type != wxPg::TYPE_INT && !change_type_loaded(field, wxPg::TYPE_INT, test_name))
        return false;
    if(type != wxPg::TYPE_FLOAT && !change_type_loaded(field, wxPg::TYPE_FLOAT, test_name))
        return false;
    if(type != wxPg::TYPE_STRING && !change_type_loaded(field, wxPg::TYPE_STRING, test_name))
        return false;
    if(type != wxPg::TYPE_DATE && !change_type_loaded(field, wxPg::TYPE_DATE, test_name))
        return false;
    if(type != wxPg::TYPE_TIME && !change_type_loaded(field, wxPg::TYPE_TIME, test_name))
        return false;
    if(type != wxPg::TYPE_TIMESTAMP && !change_type_loaded(field, wxPg::TYPE_TIMESTAMP, test_name))
        return false;
    if(type != wxPg::TYPE_BOOL && !change_type_loaded(field, wxPg::TYPE_BOOL, test_name))
        return false;

    return true;
}


bool
run_test_type_loaded(){
    testee.unload();
    if(testee.loaded() || testee.cols()){
        logger.log_error("Fatal error", "Failed to prepare clean testee.");
        return false;
    }
    testee.table(wxT("updatable_row_test"));
    if(!testee.load(wxT("id = 1"))){
        wxString msg = wxT("failed to load.") + testee.last_err();
        logger.log_error("Fatal error", msg.char_str());
        return false;
    }

    if(!change_field_type(wxT("id"), "type loaded"))
        return false;

    logger.log_success("type loaded");
    return true;
}


bool
check_insert(int id, int bool_val){
    if(!testee.insert()){
        wxString msg = wxT("insert failed: ") + testee.last_err();
        logger.log_error("insert", msg.char_str());
        return false;
    }

    wxString condition = wxT("id = ");
    condition << id;
    testee.unload();
    testee.load(condition);
    if(!check_field_int(wxT("id"), id, "insert"))
        return false;
    if(!check_field_float(wxT("fnum"), 10.1, "insert"))
        return false;
    if(!check_field_str(wxT("title"), wxT("bbb"), "insert"))
        return false;
    wxDateTime dt;
    dt.ParseDateTime(wxT("2009-08-09 20:21:22"));
    if(!check_field_date(wxT("create_date"), dt, "insert"))
        return false;
    if(!check_field_time(wxT("create_time"), dt, "insert"))
        return false;
    if(!check_field_timestamp(wxT("create_timestamp"), dt, "insert"))
        return false;
    if(bool_val < 0){
        if(!check_field_null(wxT("ok"), "insert"))
            return false;
    }else if(!check_field_bool(wxT("ok"), bool_val > 0, "insert"))
        return false;

    return true;
}


bool
run_test_insert(){
    testee.unload();
    testee.table(wxT("updatable_row_test"));
    testee.set_value(wxT("id"), 2);
    testee.set_value(wxT("fnum"), 10.1);
    testee.set_value(wxT("title"), wxT("bbb"));
    wxDateTime dt;
    dt.ParseDateTime(wxT("2009-08-09 20:21:22"));
    testee.set_value(wxT("create_timestamp"), dt);
    testee.set_date(wxT("create_date"), dt);
    testee.set_time(wxT("create_time"), dt);
    testee.set_null(wxT("ok"));
    if(!check_insert(2, -1))
        return false;

    testee.set_value(wxT("id"), 3);
    testee.set_bool(wxT("ok"), true);
    if(!check_insert(3, 1))
        return false;

    testee.set_value(wxT("id"), 4);
    testee.set_bool(wxT("ok"), false);
    if(!check_insert(4, 0))
        return false;

    logger.log_success("insert");
    return true;
}


bool
run_test_update(){
    if(!db.run(wxT("insert into updatable_row_test values(1000, 'aaa', 2.5, \
                    '2005-05-05', '15:16:17', '2005-05-05 15:16:17', true)"))){
        logger.log_error("Fatal error", "Failed to insert data");
        return false;
    }
    testee.unload();
    testee.table(wxT("updatable_row_test"));
    testee.set_value(wxT("id"), 1001);
    testee.set_value(wxT("fnum"), 10.1);
    testee.set_value(wxT("title"), wxT("bbb"));
    wxDateTime dt;
    dt.ParseDateTime(wxT("2009-08-09 20:21:22"));
    testee.set_value(wxT("create_timestamp"), dt);
    testee.set_date(wxT("create_date"), dt);
    testee.set_time(wxT("create_time"), dt);
    testee.set_null(wxT("ok"));
    if(!testee.update(wxT("id = 1000"))){
        wxString msg = wxT("update failed: ") + testee.last_err();
        logger.log_error("update unknown", msg.char_str());
        return false;
    }
    if(!testee.load(wxT("id = 1001"))){
        wxString msg = wxT("post-update load failed: ") + testee.last_err();
        logger.log_error("update unknown", msg.char_str());
        return false;
    }
    if(!check_field_int(wxT("id"), 1001, "update unknown"))
        return false;
    if(!check_field_float(wxT("fnum"), 10.1, "update unknown"))
        return false;
    if(!check_field_str(wxT("title"), wxT("bbb"), "update unknown"))
        return false;
    if(!check_field_date(wxT("create_date"), dt, "update unknown"))
        return false;
    if(!check_field_time(wxT("create_time"), dt, "update unknown"))
        return false;
    if(!check_field_timestamp(wxT("create_timestamp"), dt, "update unknown"))
        return false;
    if(!check_field_null(wxT("ok"), "update unknown"))
        return false;
    logger.log_success("update unknown");

    testee.set_value(wxT("id"), 1000);
    testee.set_bool(wxT("ok"), true);
    if(!testee.update()){
        wxString msg = wxT("update failed: ") + testee.last_err();
        logger.log_error("update loaded", msg.char_str());
        return false;
    }
    if(!testee.load(wxT("id = 1000"))){
        wxString msg = wxT("post-update load failed: ") + testee.last_err();
        logger.log_error("update loaded", msg.char_str());
        return false;
    }
    if(!check_field_int(wxT("id"), 1000, "update loaded"))
        return false;
    if(!check_field_bool(wxT("ok"), true, "update loaded"))
        return false;
    logger.log_success("update loaded");

    return true;
}


bool
run_test_delete_record(){
    if(!db.run(wxT("insert into updatable_row_test values(10000, 'aaa', 2.5, \
                    '2005-05-05', '15:16:17', '2005-05-05 15:16:17', true)"))){
        logger.log_error("Fatal error", "Failed to insert data");
        return false;
    }
    testee.unload();
    testee.table(wxT("updatable_row_test"));
    if(!testee.load(wxT("id = 10000"))){
        wxString msg = wxT("load failed: ") + testee.last_err();
        logger.log_error("Fatal error", msg.char_str());
        return false;
    }
    if(!testee.delete_record()){
        wxString msg = wxT("delete failed: ") + testee.last_err();
        logger.log_error("delete_record", msg.char_str());
        return false;
    }
    testee.unload();
    if(!testee.load(wxT("id = 10000"))){
        wxString msg = wxT("post-delete load failed: ") + testee.last_err();
        logger.log_error("Fatal error", msg.char_str());
        return false;
    }
    if(!check_field_null(wxT("id"), "delete_record"))
        return false;
    if(!check_field_null(wxT("fnum"), "delete_record"))
        return false;
    if(!check_field_null(wxT("title"), "delete_record"))
        return false;
    if(!check_field_null(wxT("create_date"), "delete_record"))
        return false;
    if(!check_field_null(wxT("create_time"), "delete_record"))
        return false;
    if(!check_field_null(wxT("create_timestamp"), "delete_record"))
        return false;
    if(!check_field_null(wxT("ok"), "delete_record"))
        return false;

    logger.log_success("delete_record");
    return true;
}


bool
run_tests(){
    bool result = true;
    if(test_setters_empty)
        result = run_test_setters_empty() && result;
    if(test_clear_empty)
        result = run_test_clear_empty() && result;
    testee.clear();
    if(test_type_empty)
        result = run_test_type_empty() && result;
    testee.unload();
    if(test_load)
        result = run_test_load() && result;
    testee.unload();
    if(test_setters_loaded)
        result = run_test_setters_loaded() && result;
    testee.unload();
    if(test_clear_loaded)
        result = run_test_clear_loaded() && result;
    testee.unload();
    if(test_unload)
        result = run_test_unload() && result;
    testee.unload();
    if(test_type_loaded)
        result = run_test_type_loaded() && result;
    testee.unload();
    if(test_insert)
        result = run_test_insert() && result;
    testee.unload();
    if(test_update)
        result = run_test_update() && result;
    testee.unload();
    if(test_delete_record)
        result = run_test_delete_record() && result;
    return result;
}


int
main(int argc, char **argv){
    if(!wxInitialize()){
        std::cerr << "Initialization of wxWidgets failed!";
        return 1;
    }

    parse_options(argc, argv);

    if(!prepare_test_env())
        return 2;

    bool result = run_tests();

    drop_test_table();
    wxUninitialize();
    logger.output_summary();

    if(result)
        return 0;
    std::cerr << "Tests failed!";
    return 3;
}
