/* list_binding.h - binding wxWindow to contain the values of some
    field in wxPg::Result_data

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __LIST_BINDING_H
#define __LIST_BINDING_H

#include "wxPg/conf.h"
#include "wxPg/binding/binding.h"
#include "wxPg/binding/binding_list.h"
#include<vector>


namespace wxPg{

/** Binding containers to contain values from entire column in
    wxPg::Result_data_provider.
*/
class wxPG_DECLSPEC
List_binding : public Binding{
public:
    /** Creates new object.
        Call data() to complete initialization.
    */
    List_binding();

    /** Creates new object.
        @param data will be used to update containers.
    */
    explicit
    List_binding(Result_data_provider *data);

    /** Destroyes the object. */
    virtual
    ~List_binding(){}

    /** Set result data, used to update containers.
        @param data - new result data to be used.
    */
    void
    data(Result_data_provider *data){this->data_ = data;}

    /** Return currently used data to update containers.
        @return currently used data to update containers.
    */
    Result_data_provider*
    data(){return this->data_;}

    /** Bind wxWindow to this object.
        Usualy you don't need to call this method directly -
        use binding macros, defined in binding_macros.h.
        @param updater - object, responsible for updating window.
    */
    void
    bind(wxWindow_list_updater *updater);

    /** Remove container from bindings.
        @param w is no longer updated.
    */
    virtual void
    unbind(wxWindow *w);

    /** Updates UI.
        For each binded container a specified function is called.
    */
    virtual void
    update_UI();
protected:
    Result_data_provider*
    data_;

    //widget bindings
    std::vector<wxWindow_list_updater*>
    ctrl_bindings_;
private:
    List_binding(const List_binding&){}
    List_binding &operator=(const List_binding&){return *this;}
};

}


#endif
