/* updatable_row_binding.h - binding wxWindow to display and edit value
    of some field in wxPg::Updatable_row.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __UPDATABLE_ROW_BINDING_H
#define __UPDATABLE_ROW_BINDING_H

#include "wxPg/conf.h"
#include "wxPg/binding/row_binding.h"
#include "wxPg/binding/binding_ctrl.h"


namespace wxPg{

/** Binding widgets to modify values from wxPg::Updatable_row. */
class wxPG_DECLSPEC
Updatable_row_binding : public Row_binding{
public:
    /** Creates new object.
        Call data() to complete initialization of object.
    */
    Updatable_row_binding(){}

    /** Creates new object.
        @param data - Updatable_row to be used.
    */
    explicit
    Updatable_row_binding(Updatable_row *data);

    /** Destroys the object. */
    virtual
    ~Updatable_row_binding(){}

    /** Returns currently used Updatable_row.
        @return currently used Updatable_row.
    */
    Updatable_row*
    data(){return dynamic_cast<Updatable_row*>(this->Row_binding::data());}

    /** Sets currently used Updatable_row.
        @param data - new Updatable_row to be used.
    */
    void
    data(Updatable_row *data){this->Row_binding::data(data);}

    /** Collect values from widgets and set fields of Updatable_row. */
    virtual void
    collect();
};

}

#endif
