/* cached_row_provider.h - abstract class wxPg::Cached_row_provider for
    accessing data from single cached row of SQL query result.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __CACHED_ROW_PROVIDER_H
#define __CACHED_ROW_PROVIDER_H

#include "wxPg/conf.h"
#include "wxPg/db/data_row_provider.h"

namespace wxPg{

//foreward declaration
class Database;

/** Abstract class, representing a single cached row of SQL query result. */
class wxPG_DECLSPEC
Cached_row_provider : virtual public Data_row_provider{
public:
    /** Destructor. */
    virtual
    ~Cached_row_provider(){}

    //do not hide parent methods
    using Data_row_provider::type_simple;
    using Data_row_provider::null;
    using Data_row_provider::get_str;

    //inherited
    virtual int
    type_simple(const wxString &field);

    //inherited
    virtual bool
    null(const wxString &field);

    //inherited
    virtual int
    get_int(int idx);

    //inherited
    virtual int
    get_int(const wxString &field);

    //inherited
    virtual double
    get_float(int idx);

    //inherited
    virtual double
    get_float(const wxString &field);

    //inherited
    virtual const wxString
    get_str(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_date(int idx);

    //inherited
    virtual const wxDateTime
    get_date(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_time(int idx);

    //inherited
    virtual const wxDateTime
    get_time(const wxString &field);

    //inherited
    virtual const wxDateTime
    get_date_time(int idx);

    //inherited
    virtual const wxDateTime
    get_date_time(const wxString &field);

    //inherited
    virtual bool
    get_bool(int idx);

    //inherited
    virtual bool
    get_bool(const wxString &field);
protected:
    /** Translate value, retrieved from database to cache value.
        Used for translating values from PQgetvalue() to wxString.
        @param value - value from database.
        @param type - data type.
        @param db - database connection.
        @return string value for cache.
    */
    virtual const wxString
    format_cached_value(const char *value, int type, const Database &db)const;
};

}

#endif
